"""Test the MCP server functionality."""

import asyncio
import base64
import os
import signal
import subprocess
import time
from pathlib import Path

import pytest
import requests
from mcp import ClientSession
from mcp.client.sse import sse_client
from mcp.client.stdio import stdio_client
from mcp.types import (
    BlobResourceContents,
    EmbeddedResource,
    ImageContent,
    PromptMessage,
    TextContent,
    TextResourceContents,
)
from pydantic import AnyUrl
from typer.testing import CliRunner

from starbridge.hello import Service as HelloService
from tests.utils_test import _server_parameters

try:
    from starbridge.hello.cli import bridge
except ImportError:
    bridge = None

MOCK_GET_ALL_SPACES = "atlassian.Confluence.get_all_spaces"
MOCK_GET_SPACE = "atlassian.Confluence.get_space"
PYPROJECT_TOML = "pyproject.toml"
DOT_COVERAGE = ".coverage"

EXPECTED_TOOLS = [
    "starbridge_claude_health",
    "starbridge_claude_info",
    "starbridge_claude_restart",
    "starbridge_confluence_health",
    "starbridge_confluence_info",
    "starbridge_confluence_page_create",
    "starbridge_confluence_page_delete",
    "starbridge_confluence_page_get",
    "starbridge_confluence_page_list",
    "starbridge_confluence_page_update",
    "starbridge_confluence_space_list",
    "starbridge_hello_health",
    "starbridge_hello_hello",
    "starbridge_hello_info",
    "starbridge_hello_pdf",
]


@pytest.fixture
def runner() -> CliRunner:
    """Get a Click CLI test runner."""
    return CliRunner()


@pytest.mark.asyncio
async def test_mcp_server_list_tools_stdio() -> None:
    """Test listing of tools from the server."""
    # Expected tool names that should be present
    expected_tools = EXPECTED_TOOLS.copy()
    if bridge:
        expected_tools.append("starbridge_hello_bridge")

    async with stdio_client(_server_parameters()) as (read, write), ClientSession(read, write) as session:
        # Initialize the connection
        await session.initialize()

        # List available tools
        result = await session.list_tools()

        # Verify each expected tool is present
        tool_names = [tool.name for tool in result.tools]
        for expected_tool in expected_tools:
            assert expected_tool in tool_names


@pytest.mark.skip(reason="SSE test disabled temporarily")
@pytest.mark.asyncio
async def test_mcp_server_list_tools_sse() -> None:
    """Test listing of tools from the server in sse mode."""
    expected_tools = EXPECTED_TOOLS.copy()

    # Start the server in SSE mode
    env = os.environ.copy()
    env.update({
        "COVERAGE_PROCESS_START": PYPROJECT_TOML,
        "COVERAGE_FILE": os.getenv("COVERAGE_FILE", DOT_COVERAGE),
        "PYTHONPATH": ".",
    })

    process = await asyncio.create_subprocess_exec(
        "uv",
        "run",
        "starbridge",
        "mcp",
        "serve",
        "--host",
        "0.0.0.0",
        "--port",
        "8002",
        env=env,
        stdout=asyncio.subprocess.PIPE,
        stderr=asyncio.subprocess.PIPE,
    )

    try:
        # Give the server a moment to start up
        await asyncio.sleep(2)

        # Connect to the server using SSE
        async with (
            sse_client(
                "http://0.0.0.0:8002/sse",
                timeout=1,
                sse_read_timeout=1,
            ) as (read, write),
            ClientSession(read, write) as session,
        ):
            await session.initialize()

            result = await session.list_tools()

            # Verify each expected tool is present
            tool_names = [tool.name for tool in result.tools]
            for expected_tool in expected_tools:
                assert expected_tool in tool_names

        process.terminate()

    finally:
        process.terminate()
        # Clean up the subprocess
        if process.returncode is None:
            process.send_signal(signal.SIGTERM)
            await process.wait()


@pytest.mark.asyncio
async def test_mcp_server_list_resources() -> None:
    """Test listing of resources from the server."""
    async with (
        stdio_client(_server_parameters([MOCK_GET_ALL_SPACES])) as (read, write),
        ClientSession(read, write) as session,
    ):
        await session.initialize()

        result = await session.list_resources()

        assert result.resources is not None
        assert len(result.resources) == 1
        assert any(
            resource.name == "helmut"
            for resource in result.resources
            if str(resource.uri) == "starbridge://confluence/space/~7120201709026d2b41448e93bb58d5fa301026"
        )


@pytest.mark.asyncio
async def test_mcp_server_read_resource() -> None:
    """Test reading a resource from the server."""
    async with (
        stdio_client(
            _server_parameters([
                MOCK_GET_ALL_SPACES,
                MOCK_GET_SPACE,
            ]),
        ) as (
            read,
            write,
        ),
        ClientSession(read, write) as session,
    ):
        await session.initialize()

        result = await session.read_resource(
            AnyUrl(
                "starbridge://confluence/space/~7120201709026d2b41448e93bb58d5fa301026",
            ),
        )
        assert len(result.contents) == 1
        content = result.contents[0]
        assert type(content) is TextResourceContents
        assert content.text == Path("tests/fixtures/get_space.json").read_text(encoding="utf-8")


@pytest.mark.asyncio
async def test_mcp_server_list_prompts() -> None:
    """Test listing of prompts from the server."""
    async with stdio_client(_server_parameters()) as (read, write), ClientSession(read, write) as session:
        await session.initialize()

        result = await session.list_prompts()

        assert result.prompts is not None


@pytest.mark.asyncio
async def test_mcp_server_prompt_get() -> None:
    """Test getting a prompt from the server."""
    async with (
        stdio_client(_server_parameters([MOCK_GET_ALL_SPACES])) as (read, write),
        ClientSession(read, write) as session,
    ):
        await session.initialize()

        result = await session.get_prompt(
            "starbridge_confluence_space_summary",
            {"style": "detailed"},
        )

        assert len(result.messages) == 1
        message = result.messages[0]
        assert type(message) is PromptMessage
        assert type(message.content) is TextContent
        assert (
            message.content.text == "Here are the current spaces to summarize: Give extensive details.\n\n"
            "- ~7120201709026d2b41448e93bb58d5fa301026: helmut (personal)"
        )


@pytest.mark.asyncio
async def test_mcp_server_tool_call() -> None:
    """Test calling a tool on the server."""
    async with stdio_client(_server_parameters()) as (read, write), ClientSession(read, write) as session:
        await session.initialize()

        result = await session.call_tool("starbridge_hello_hello", {})
        assert len(result.content) == 1
        content = result.content[0]
        assert type(content) is TextContent
        assert content.text == "Hello World!"

        result = await session.call_tool(
            "starbridge_hello_hello",
            {"locale": "de_DE"},
        )
        assert len(result.content) == 1
        content = result.content[0]
        assert type(content) is TextContent
        assert content.text == "Hallo Welt!"


if hasattr(HelloService, "bridge"):  # if extra imaging

    @pytest.mark.asyncio
    async def test_mcp_server_tool_call_with_image() -> None:
        """Test calling a tool with image content on the server."""
        async with stdio_client(_server_parameters()) as (read, write), ClientSession(read, write) as session:
            await session.initialize()

            result = await session.call_tool("starbridge_hello_bridge", {})
            assert len(result.content) == 1
            content = result.content[0]
            assert type(content) is ImageContent
            assert content.data == base64.b64encode(
                Path("tests/fixtures/starbridge.png").read_bytes(),
            ).decode("utf-8")


@pytest.mark.asyncio
async def test_mcp_server_tool_call_with_pdf() -> None:
    """Test calling a tool with PDF content on the server."""
    async with stdio_client(_server_parameters()) as (read, write), ClientSession(read, write) as session:
        await session.initialize()

        result = await session.call_tool("starbridge_hello_pdf", {})
        assert len(result.content) == 1
        content = result.content[0]
        assert type(content) is EmbeddedResource
        assert type(content.resource) is BlobResourceContents
        assert content.resource.mimeType == "application/pdf"
        assert content.resource.blob == base64.b64encode(
            Path("tests/fixtures/starbridge.pdf").read_bytes(),
        ).decode("utf-8")


@pytest.mark.skip(reason="test_core_env_args_passed disabled temporarily")
def test_mcp_server_sse_terminates(runner) -> None:
    """Test if SSE server terminates correctly."""
    env = os.environ.copy()
    env.update({
        "COVERAGE_PROCESS_START": PYPROJECT_TOML,
        "COVERAGE_FILE": os.getenv("COVERAGE_FILE", DOT_COVERAGE),
        "MOCKS": "webbrowser.open",
    })

    process = subprocess.Popen(
        [
            "uv",
            "run",
            "starbridge",
            "mcp",
            "serve",
            "--host",
            "0.0.0.0",
            "--port",
            "9000",
        ],
        text=True,
        env=env,
    )

    try:
        # Give the server time to start
        time.sleep(10)

        # Send terminate request
        try:
            response = requests.get("http://0.0.0.0:9000/terminate", timeout=10)
            response.raise_for_status()
        except Exception:  # noqa: S110, BLE001
            pass

        # Wait for process to end (timeout after 5 seconds)
        process.wait(timeout=10)

        assert process.returncode == 0

    finally:
        process.terminate()
        process.kill()



"""Tests for platform-specific functionality."""

import json
import os
import subprocess

from starbridge import __is_running_in_container__


def test_container_running_in_platform() -> None:
    """Check behavior of container running in platform."""
    assert not __is_running_in_container__

    env = os.environ.copy()
    env["STARBRIDGE_RUNNING_IN_CONTAINER"] = "1"

    result = subprocess.run(
        ["uv", "run", "starbridge", "info"],
        capture_output=True,
        text=True,
        env=env,
        check=False,
    )

    info = json.loads(result.stdout)
    assert info["is_running_in_container"] is True



"""Data models for MCP functionality."""

from dataclasses import dataclass


@dataclass(frozen=True)
class ResourceMetadata:
    """
    A resource type is identified by a triple of (server, service, type).

    Args:
        server (str): The server name.
        service (str): The service name.
        type (str): The resource type.

    """

    server: str
    service: str
    type: str

    def __str__(self) -> str:
        """
        Convert resource metadata to string representation.

        Returns:
            str: String representation in format "server://service/type"

        """
        return f"{self.server}://{self.service}/{self.type}"


@dataclass(frozen=True)
class PromptMetadata:
    """
    A prompt is identified by server, service, and type.

    Args:
        server (str): The server name.
        service (str): The service name.
        type (str): The prompt type.

    """

    server: str
    service: str
    type: str

    def __str__(self) -> str:
        """
        Convert prompt metadata to string representation.

        Returns:
            str: String representation in format "server_service_type"

        """
        return f"{self.server}_{self.service}_{self.type}"


@dataclass(frozen=True)
class ToolMetadata:
    """
    A tool is identified by server, service, and name.

    Args:
        server (str): The server name.
        service (str): The service name.
        name (str): The tool name.

    """

    server: str
    service: str
    name: str

    def __str__(self) -> str:
        """
        Convert tool metadata to string representation.

        Returns:
            str: String representation in format "server_service_name"

        """
        return f"{self.server}_{self.service}_{self.name}"



"""Test search CLI functionality."""

import json
from unittest.mock import patch

import httpx
import pytest
import requests
from brave_search_python_client import MOCK_API_KEY, WebSearchApiResponse
from typer.testing import CliRunner

from starbridge.cli import cli


@pytest.fixture
def runner() -> CliRunner:
    """Get a Click CLI test runner."""
    return CliRunner()


def test_search_cli_info(runner) -> None:
    """Check search info."""
    result = runner.invoke(cli, ["search", "info"])
    assert result.exit_code == 0


def test_search_cli_health(runner) -> None:
    """Check search health."""
    result = runner.invoke(cli, ["search", "health"])
    assert '"UP"' in result.output
    assert result.exit_code == 0


@patch("httpx.AsyncClient.head")
def test_search_cli_health_not_connected(mock_head, runner) -> None:
    """Check web health down when not connected."""
    mock_head.side_effect = httpx.ReadTimeout("Timeout")

    result = runner.invoke(cli, ["search", "health"])
    assert '"DOWN"' in result.output
    assert result.exit_code == 0


def test_search_cli_web(runner) -> None:
    """Check searching the web."""
    with patch.dict(
        "os.environ",
        {"STARBRIDGE_SEARCH_BRAVE_SEARCH_API_KEY": MOCK_API_KEY},
    ):
        result = runner.invoke(
            cli,
            [
                "search",
                "web",
                "hello world",
            ],
        )
        assert result.exit_code == 0
        response = WebSearchApiResponse.model_validate(json.loads(result.output))
        assert response.web
        assert response.web.results
        assert len(response.web.results) > 0


@patch("httpx.AsyncClient.get")
def test_search_cli_get_timeouts(mock_get, runner) -> None:
    """Check getting content fails."""
    mock_get.side_effect = requests.exceptions.Timeout()

    with patch.dict("os.environ", {"STARBRIDGE_SEARCH_BRAVE_SEARCH_API_KEY": "OTHER"}):
        result = runner.invoke(
            cli,
            [
                "search",
                "web",
                "hello world",
            ],
        )
        assert "Request failed" in result.output
        assert result.exit_code == 1



"""Settings used for searching."""

from typing import Annotated

from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict

from starbridge import __project_name__


class Settings(BaseSettings):
    """Settings for search module."""

    model_config = SettingsConfigDict(
        env_prefix=f"{__project_name__.upper()}_SEARCH_",
        extra="ignore",
        env_file=".env",
        env_file_encoding="utf-8",
    )

    brave_search_api_key: Annotated[
        str,
        Field(
            ...,
            description="Brave Search API Key (see https://brave.com/search/api/).",
        ),
    ]



"""Tests for Hello World CLI functionality."""

from pathlib import Path
from unittest.mock import patch

import pytest
from typer.testing import CliRunner

from starbridge.cli import cli

try:
    from starbridge.hello.cli import bridge
except ImportError:
    bridge = None


@pytest.fixture
def runner() -> CliRunner:
    """Get a Click CLI test runner."""
    return CliRunner()


def test_hello_cli_info(runner) -> None:
    """Check hello info."""
    result = runner.invoke(cli, ["hello", "info"])
    assert "en_US" in result.output
    assert result.exit_code == 0


def test_hello_cli_health(runner) -> None:
    """Check hello health."""
    result = runner.invoke(cli, ["hello", "health"])
    assert '"UP"' in result.output
    assert result.exit_code == 0


if bridge:  # if extra imaging

    def test_hello_cli_bridge(runner) -> None:
        """Check we dump the image."""
        with runner.isolated_filesystem():
            result = runner.invoke(cli, ["hello", "bridge", "--dump"])
            assert result.exit_code == 0
            assert Path("starbridge.png").is_file()
            assert Path("starbridge.png").stat().st_size == 6235

    @patch("cairosvg.svg2png", side_effect=OSError)
    def test_hello_cli_bridge_error(mock_svg2png, runner) -> None:
        """Check we handle cairo missing."""
        result = runner.invoke(cli, ["hello", "bridge"])
        assert result.exit_code == 78

else:

    @pytest.mark.no_extras
    def test_hello_cli_no_imaging_extra_no_bridge(runner) -> None:
        """Check we handle missing PIL."""
        result = runner.invoke(cli, ["hello"])
        assert "Show image" not in result.output
        assert result.exit_code == 0


def test_hello_cli_pdf(runner) -> None:
    """Check we dump the pdf."""
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ["hello", "pdf", "--dump"])
        assert result.exit_code == 0
        assert Path("starbridge.pdf").is_file()
        assert Path("starbridge.pdf").stat().st_size == 6840


@patch("sys.platform", new="linux")
@patch("subprocess.run")
def test_hello_cli_pdf_open(mock_run, runner) -> None:
    """Check we open the pdf."""
    with runner.isolated_filesystem():
        result = runner.invoke(cli, ["hello", "pdf"])
        assert result.exit_code == 0

        # Verify xdg-open was called
        mock_run.assert_called_once()
        args, kwargs = mock_run.call_args
        assert args[0][0] == "xdg-open"
        assert str(args[0][1]).endswith(".pdf")
        assert kwargs["check"] is True



"""
MCP decorators module providing function decorators for registering MCP capabilities.

This module contains decorators for registering tools, resources, and prompts in the
Starbridge server.
"""

from collections.abc import Callable
from functools import wraps
from typing import ParamSpec, TypeVar

from starbridge import __project_name__

from .models import PromptMetadata, ResourceMetadata, ToolMetadata

P = ParamSpec("P")
R = TypeVar("R")

"""Decorators for registering MCP capabilities like tools, resources and prompts."""


def mcp_tool(
    server: str = __project_name__,
    service: str | None = None,
    name: str | None = None,
) -> Callable[[Callable[P, R]], Callable[P, R]]:
    """
    Mark a method as an MCP tool.

    Args:
        server (str): The server name. Defaults to "starbridge".
        service (str): The service name. If not provided, derived from module name.
        name (str | None): The tool name. If not provided, derived from function name.

    Returns:
        Callable: Decorated function that registers as MCP tool.

    """

    def decorator(func: Callable[P, R]) -> Callable[P, R]:
        @wraps(func)
        def wrapper(*args, **kwargs) -> R:
            return func(*args, **kwargs)

        wrapper.__mcp_tool__ = ToolMetadata(  # type: ignore
            server=server,
            service=service or _get_service_name(func),
            name=name or func.__name__,
        )
        return wrapper

    return decorator


def mcp_resource_iterator(
    server: str = __project_name__,
    service: str | None = None,
    type: str | None = None,  # noqa: A002
) -> Callable:
    """
    Mark a method as a resource iterator.

    Args:
        server (str): The server name. Defaults to project name.
        service (str | None): The service name. Defaults to module name if not provided.
        type (str | None): The resource type.

    Returns:
        Callable: Decorated function that registers as a resource iterator.

    Raises:
        ValueError: If resource type is missing or duplicated.

    """

    def decorator(func: Callable) -> Callable:
        @wraps(func)
        def wrapper(*args, **kwargs) -> R:  # type: ignore
            return func(*args, **kwargs)

        wrapper.__mcp_resource_iterator__ = ResourceMetadata(  # type: ignore
            server=server,
            service=service or _get_service_name(func),
            type=type or "",
        )
        return wrapper

    return decorator


def mcp_resource(
    server: str = __project_name__,
    service: str | None = None,
    type: str | None = None,  # noqa: A002
) -> Callable:
    """
    Mark a method as a resource handler.

    Args:
        server (str): The server name. Defaults to project name.
        service (str | None): The service name. Defaults to module name if not provided.
        type (str | None): The resource type.

    Returns:
        Callable: Decorated function that registers as a resource handler.

    """

    def decorator(func: Callable) -> Callable:
        @wraps(func)
        def wrapper(*args, **kwargs) -> R:  # type: ignore
            return func(*args, **kwargs)

        wrapper.__mcp_resource__ = ResourceMetadata(  # type: ignore
            server=server,
            service=service or _get_service_name(func),
            type=type or "",
        )
        return wrapper

    return decorator


def mcp_prompt(
    server: str = __project_name__,
    service: str | None = None,
    type: str | None = None,  # noqa: A002
) -> Callable:
    """
    Mark a method as a prompt handler.

    Args:
        server (str): The server name. Defaults to project name.
        service (str | None): The service name. Defaults to module name if not provided.
        type (str | None): The prompt type. Defaults to method name without `prompt_` prefix.

    Returns:
        Callable: Decorated function that registers as a prompt handler.

    """

    def decorator(func: Callable) -> Callable:
        @wraps(func)
        def wrapper(*args, **kwargs) -> R:  # type: ignore
            return func(*args, **kwargs)

        wrapper.__mcp_prompt__ = PromptMetadata(  # type: ignore
            server=server,
            service=service or _get_service_name(func),
            type=type or func.__name__.replace("prompt_", ""),
        )
        return wrapper

    return decorator


def _get_service_name(func: Callable) -> str:
    """
    Extract service name from function's module path.

    Args:
        func (Callable): The function to extract module name from

    Returns:
        str: The extracted service name

    """
    return func.__module__.split(".")[1]



"""Common test fixtures and configuration."""

from pathlib import Path

import pytest


@pytest.fixture(scope="session")
def docker_compose_file(pytestconfig) -> str:
    """Get the path to the docker compose file."""
    return str(Path(pytestconfig.rootdir) / "compose.yaml")



"""Platform and process related utilities."""

from pathlib import Path

import psutil
from pydantic import BaseModel


class ParentProcessInfo(BaseModel):
    """Information about a parent process."""

    name: str | None = None
    pid: int | None = None


class ProcessInfo(BaseModel):
    """Information about the current process."""

    project_root: str
    pid: int
    parent: ParentProcessInfo


def get_process_info() -> ProcessInfo:
    """
    Get information about the current process and its parent.

    Returns:
        ProcessInfo: Object containing process information.

    """
    current_process = psutil.Process()
    parent = current_process.parent()

    return ProcessInfo(
        project_root=str(Path(__file__).parent.parent.parent.parent),
        pid=current_process.pid,
        parent=ParentProcessInfo(
            name=parent.name() if parent else None,
            pid=parent.pid if parent else None,
        ),
    )



"""Test search MCP functionality."""

import pytest
from mcp import ClientSession
from mcp.client.stdio import stdio_client
from mcp.types import TextContent
from typer.testing import CliRunner

from tests.utils_test import _server_parameters

PYPROJECT_TOML = "pyproject.toml"
DOT_COVERAGE = ".coverage"


@pytest.fixture
def runner() -> CliRunner:
    """Get a Click CLI test runner."""
    return CliRunner()


@pytest.mark.asyncio
async def test_search_mcp_tool_web() -> None:
    """Test server tool search."""
    async with stdio_client(_server_parameters()) as (read, write), ClientSession(read, write) as session:
        await session.initialize()

        result = await session.call_tool(
            "starbridge_search_web",
            {"q": "hello world"},
        )
        assert len(result.content) == 1
        content = result.content[0]
        assert type(content) is TextContent
        assert "hello" in content.text



"""Tests for web CLI functionality."""

import json
from unittest.mock import patch

import httpx
import pytest
from typer.testing import CliRunner

from starbridge.cli import cli
from starbridge.web import GetResult

GET_TEST_HTML_URL = "https://starbridge.readthedocs.io/en/latest/"
GET_LLMS_TXT_URL = "https://docs.anthropic.com"


@pytest.fixture
def runner() -> CliRunner:
    """Get a Click CLI test runner."""
    return CliRunner()


def test_web_cli_info(runner) -> None:
    """Check web info."""
    result = runner.invoke(cli, ["web", "info"])
    assert result.exit_code == 0


def test_web_cli_health(runner) -> None:
    """Check web health."""
    result = runner.invoke(cli, ["web", "health"])
    assert '"UP"' in result.output
    assert result.exit_code == 0


@patch("httpx.head")
def test_web_cli_health_not_connected(mock_head, runner) -> None:
    """Check web health down when not connected."""
    mock_head.side_effect = httpx.TimeoutException("timeout")

    result = runner.invoke(cli, ["web", "health"])
    assert '"DOWN"' in result.output
    assert result.exit_code == 0


@patch("httpx.AsyncClient.get")
def test_web_cli_get_timeouts(mock_get, runner) -> None:
    """Check getting content fails."""
    mock_get.side_effect = httpx.TimeoutException("timeout")

    result = runner.invoke(
        cli,
        [
            "web",
            "get",
            GET_TEST_HTML_URL,
        ],
    )
    assert "Failed to fetch robots.txt" in result.output
    assert result.exit_code == 1


def test_web_cli_get_html_no_transform_works(runner) -> None:
    """Check getting content from the web as html encoded in unicode."""
    result = runner.invoke(
        cli,
        [
            "web",
            "get",
            "--no-transform-to-markdown",
            GET_TEST_HTML_URL,
        ],
    )
    rtn = GetResult.model_validate(json.loads(result.output))
    assert (rtn.resource.text or "").startswith("<!doctype html>")
    assert rtn.get_link_count() > 0
    assert result.exit_code == 0


def test_web_cli_get_html_no_transform_no_links(runner) -> None:
    """Check getting content from the web as html encoded in unicode, without extracting links."""
    result = runner.invoke(
        cli,
        [
            "web",
            "get",
            "--no-transform-to-markdown",
            "--no-extract-links",
            GET_TEST_HTML_URL,
        ],
    )
    rtn = GetResult.model_validate(json.loads(result.output))
    assert (rtn.resource.text or "").startswith("<!doctype html>")
    assert rtn.get_link_count() == 0
    assert result.exit_code == 0


def test_web_cli_get_html_to_markdown(runner) -> None:
    """Check getting content from the web as markdown."""
    result = runner.invoke(
        cli,
        [
            "web",
            "get",
            GET_TEST_HTML_URL,
        ],
    )
    rtn = GetResult.model_validate(json.loads(result.output))
    assert "starbridge " in (rtn.resource.text or "")
    assert result.exit_code == 0


def test_web_cli_get_french(runner) -> None:
    """Check getting content from the web in french."""
    result = runner.invoke(
        cli,
        [
            "web",
            "get",
            "--accept-language",
            "fr_FR",
            "https://www.google.com",
        ],
    )
    rtn = GetResult.model_validate(json.loads(result.output))
    assert "Recherche" in (rtn.resource.text or "")
    assert result.exit_code == 0


def test_web_cli_get_additional_context_llms_text(runner) -> None:
    """Check getting additional context."""
    result = runner.invoke(
        cli,
        [
            "web",
            "get",
            GET_LLMS_TXT_URL,
        ],
    )
    rtn = GetResult.model_validate(json.loads(result.output))
    llms_txt = rtn.get_context_by_type("llms_txt")
    assert llms_txt is not None
    assert "Send a structured list of input messages" in llms_txt.text
    assert len(llms_txt.text) < 400 * 1024
    assert result.exit_code == 0
    invalid_context = rtn.get_context_by_type("invalid")
    assert invalid_context is None


def test_web_cli_get_additional_context_llms_full_txt(runner) -> None:
    """Check getting additional context."""
    result = runner.invoke(
        cli,
        [
            "web",
            "get",
            "--llms-full-txt",
            GET_LLMS_TXT_URL,
        ],
    )
    rtn = GetResult.model_validate(json.loads(result.output))
    llms_txt = rtn.get_context_by_type("llms_txt")
    assert llms_txt is not None
    assert "knowledge base was last updated in August 2023" in llms_txt.text
    assert len(llms_txt.text) > 400 * 1024
    assert result.exit_code == 0


def test_web_cli_get_additional_context_not(runner) -> None:
    """Check not getting additional content."""
    result = runner.invoke(
        cli,
        [
            "web",
            "get",
            "--no-additional-context",
            GET_LLMS_TXT_URL,
        ],
    )
    rtn = GetResult.model_validate(json.loads(result.output))
    assert rtn.additional_context is None
    assert result.exit_code == 0
    llms_context = rtn.get_context_by_type("llms_txt")
    assert llms_context is None


def test_web_cli_get_forbidden_respected(runner) -> None:
    """Check getting content where robots.txt disallows fails."""
    result = runner.invoke(
        cli,
        [
            "web",
            "get",
            "https://github.com/search/advanced",
        ],
    )
    assert "robots.txt disallows crawling" in result.output
    assert result.exit_code == 1


def test_web_cli_get_forbidden_ignored(runner) -> None:
    """Check getting content where robots.txt disallows fails."""
    result = runner.invoke(
        cli,
        [
            "web",
            "get",
            "--force-not-respecting-robots-txt",
            "https://github.com/search/advanced",
        ],
    )
    assert "Where software is built" in result.output
    assert result.exit_code == 0



"""Utility functions and classes for the starbridge package."""

from .cli import prepare_cli
from .console import console
from .di import locate_implementations, locate_subclasses
from .health import AggregatedHealth, Health
from .logging import LoggingSettings, get_logger
from .platform import get_process_info
from .settings import get_starbridge_env, load_settings, prompt_for_env
from .signature import description_and_params

__all__ = [
    "AggregatedHealth",
    "Health",
    "LoggingSettings",
    "console",
    "description_and_params",
    "get_logger",
    "get_process_info",
    "get_starbridge_env",
    "load_settings",
    "locate_implementations",
    "locate_subclasses",
    "prepare_cli",
    "prompt_for_env",
]



"""
MCP Server implementation for Starbridge.

This module provides the main server implementation for Model Context Protocol,
including HTTP and stdio transport handling, service registration, and request routing.
"""

import asyncio
import base64
import json
import os
import signal
from io import BytesIO
from typing import Any
from urllib.parse import ParseResult, urlparse

import mcp.server.stdio
import pydantic_core
from mcp import types
from mcp.server import NotificationOptions, Server
from mcp.server.models import InitializationOptions
from mcp.server.sse import SseServerTransport
from mcp.types import (
    EmbeddedResource,
    ImageContent,
    TextContent,
)
from starlette.requests import Request as StarletteRequest

try:
    from PIL import Image

    _has_imaging = True
except ImportError:
    _has_imaging = False

from pydantic import AnyUrl
from starlette.applications import Starlette
from starlette.responses import PlainTextResponse
from starlette.routing import Route

from starbridge import __project_name__, __version__
from starbridge.mcp.context import MCPContext
from starbridge.mcp.decorators import mcp_tool
from starbridge.mcp.models import ResourceMetadata
from starbridge.mcp.service import MCPBaseService
from starbridge.utils import AggregatedHealth, get_logger, locate_subclasses

logger = get_logger(__name__)


class MCPServer:  # noqa: PLR0904
    """MCP Server for Starbridge."""

    def __init__(self) -> None:
        """Dynamically locate and register services."""
        self._services = []
        for service_class in MCPServer.service_classes():
            self._services.append(service_class())

        self._server = Server(__project_name__)
        self._server.list_prompts()(self.prompt_list)
        self._server.get_prompt()(self.prompt_get)
        self._server.list_resources()(self.resource_list)
        self._server.read_resource()(self.resource_get)
        self._server.list_tools()(self.tool_list)
        self._server.call_tool()(self.tool_call)

    @mcp_tool()
    def health(self, context: MCPContext | None = None) -> AggregatedHealth:  # noqa: ARG002
        """
        Get health status of all services and their dependencies.

        Returns:
            AggregatedHealth: Combined health status of all services.

        """
        dependencies = {}
        for service in self._services:
            service_name = service.__class__.__module__.split(".")[1]
            dependencies[service_name] = service.health()

        return AggregatedHealth(dependencies=dependencies)

    def get_context(self) -> MCPContext:
        """
        Get a context object for the current request.

        Returns:
            MCPContext: Context object for current request.

        Note:
            Context will only be valid during a request.
            Outside a request, most methods will error.

        """
        try:
            request_context = self._server.request_context
        except LookupError:
            request_context = None
        return MCPContext(request_context=request_context, mcp=self)

    async def resource_list(self) -> list[types.Resource]:
        """
        List all available resources across services.

        Returns:
            list[types.Resource]: List of available resources.

        """
        resources = []
        for service in self._services:
            result = service.resource_list(context=self.get_context())
            if asyncio.iscoroutine(result):
                result = await result
            resources.extend(result)
        return resources

    def _find_resource_handler(self, parsed_uri: ParseResult) -> tuple[MCPBaseService | None, Any]:
        """
        Find the appropriate resource handler for a parsed URI.

        Args:
            parsed_uri (ParseResult): Parsed URL to find handler for.

        Returns:
            tuple: Service instance and handler method.

        """
        for service in self._services:
            for method_name in dir(service.__class__):
                method = getattr(service.__class__, method_name)
                if not hasattr(method, "__mcp_resource__"):
                    continue

                meta = method.__mcp_resource__
                if (
                    meta.server == parsed_uri.scheme
                    and meta.service == parsed_uri.netloc
                    and parsed_uri.path.startswith(f"/{meta.type}/")
                ):
                    return service, method
        return None, None

    async def resource_get(self, uri: AnyUrl) -> str:
        """
        Get a resource from any service that can handle it.

        Args:
            uri (AnyUrl): URI of the resource to get.

        Returns:
            str: The resource content.

        Raises:
            ValueError: If no service is found to handle the URI.

        """
        parsed = urlparse(str(uri))
        service, handler = self._find_resource_handler(parsed)

        if handler:
            result = handler(
                service,
                parsed.path.split("/")[-1],
                context=self.get_context(),
            )
            if asyncio.iscoroutine(result):
                result = await result
            if result is not None:
                return result

        msg = f"No service found for URI: {uri}"
        raise ValueError(msg)

    async def prompt_list(self) -> list[types.Prompt]:
        """
        Get all available prompts across services.

        Returns:
            list[types.Prompt]: List of available prompts.

        """
        prompts = []
        for service in self._services:
            result = service.prompt_list(context=self.get_context())
            if asyncio.iscoroutine(result):
                result = await result
            prompts.extend(result)
        return prompts

    def _find_prompt_handler(self, name: str) -> tuple[MCPBaseService | None, Any]:
        """
        Find the appropriate prompt handler for a given name.

        Args:
            name: Full name of the prompt to find.

        Returns:
            tuple[MCPBaseService | None, Any]: Service instance and handler method, or None if not found.

        """
        server, service, prompt_type = name.split("_", 2)

        for service_instance in self._services:
            for method_name in dir(service_instance.__class__):
                method = getattr(service_instance.__class__, method_name)
                if not hasattr(method, "__mcp_prompt__"):
                    continue

                meta = method.__mcp_prompt__
                if meta.server == server and meta.service == service and meta.type == prompt_type:
                    return service_instance, method
        return None, None

    async def prompt_get(
        self,
        name: str,
        arguments: dict[str, str] | None,
    ) -> types.GetPromptResult:
        """
        Get a prompt by its full name (server_service_type).

        Args:
            name: Full prompt name in format server_service_type
            arguments: Optional arguments to pass to the prompt

        Returns:
            types.GetPromptResult: The prompt result containing description and messages

        """
        service_instance, method = self._find_prompt_handler(name)

        if method:
            if arguments:
                arguments = arguments.copy()
                arguments.pop("context", None)
                result = method(
                    service_instance,
                    **arguments,
                    context=self.get_context(),
                )
            else:
                result = method(service_instance, context=self.get_context())
            if asyncio.iscoroutine(result):
                result = await result
            return result

        return types.GetPromptResult(description=None, messages=[])

    async def tool_list(self) -> list[types.Tool]:
        """
        Get all available tools across services.

        Returns:
            list[types.Tool]: List of available tools.

        """
        tools = []
        for service in self._services:
            result = service.tool_list(context=self.get_context())
            if asyncio.iscoroutine(result):
                result = await result
            tools.extend(result)
        return tools

    def _find_tool_handler(self, name: str) -> tuple[MCPBaseService | None, Any]:
        """
        Find the appropriate tool handler for a given name.

        Args:
            name: Name of the tool to find

        Returns:
            tuple[MCPBaseService | None, Any]: Service instance and handler method, or None if not found

        """
        server, service, tool_name = name.split("_", 2)

        for service_instance in self._services:
            for method_name in dir(service_instance.__class__):
                method = getattr(service_instance.__class__, method_name)
                if not hasattr(method, "__mcp_tool__"):
                    continue

                meta = method.__mcp_tool__
                if meta.server == server and meta.service == service and meta.name == tool_name:
                    return service_instance, method
        return None, None

    async def tool_call(
        self,
        name: str,
        arguments: dict | None,
    ) -> list[types.TextContent | types.ImageContent | types.EmbeddedResource]:
        """
        Execute an MCP tool by name.

        Args:
            name: Name of the tool to call
            arguments: Optional arguments to pass to the tool

        Returns:
            list[types.TextContent | types.ImageContent | types.EmbeddedResource]: Tool execution results

        Raises:
            ValueError: If tool is not found

        """
        service_instance, method = self._find_tool_handler(name)

        if method:
            if arguments:
                arguments = arguments.copy()
                arguments.pop("context", None)
                result = method(
                    service_instance,
                    **arguments,
                    context=self.get_context(),
                )
            else:
                result = method(service_instance, context=self.get_context())
            if asyncio.iscoroutine(result):
                result = await result
            return MCPServer._marshal_result(result)

        msg = f"Unknown tool: {name}"
        raise ValueError(msg)

    async def resource_type_list(self) -> set[ResourceMetadata]:
        """
        Get all available resource types across all services.

        Returns:
            set[ResourceMetadata]: Set of resource metadata from all services.

        """
        types = set()
        for service in self._services:
            result = service.resource_type_list(context=self.get_context())
            if asyncio.iscoroutine(result):
                result = await result
            types.update(result)
        return types

    @staticmethod
    def resource_types() -> list[str]:
        """
        Get all available resource types as formatted strings.

        Returns:
            list[str]: Sorted list of resource types as formatted strings.

        """
        return sorted(str(rt) for rt in asyncio.run(MCPServer().resource_type_list()))

    def starlette_app(self, debug: bool = True) -> Starlette:
        """
        Create Starlette ASGI application for serving MCP over HTTP.

        Args:
            debug: Enable debug mode

        Returns:
            Starlette: Configured ASGI application

        """
        sse = SseServerTransport("/messages")

        async def handle_sse(request: StarletteRequest) -> None:
            async with sse.connect_sse(
                request.scope,
                request.receive,
                request._send,  # noqa: SLF001
            ) as streams:
                await self._server.run(
                    streams[0],
                    streams[1],
                    self._create_initialization_options(),
                )

        async def handle_messages(request: StarletteRequest) -> None:
            await sse.handle_post_message(request.scope, request.receive, request._send)  # noqa: SLF001

        def handle_health(request: StarletteRequest) -> PlainTextResponse:  # noqa: ARG001
            return PlainTextResponse(
                headers={"content-type": "application/json"},
                content=json.dumps(
                    self.health().model_dump(),
                ),  # Use json.dumps instead of model_dump_json
            )

        def handle_terminate(request: StarletteRequest) -> None:  # noqa: ARG001
            os.kill(os.getpid(), signal.SIGINT)
            os.kill(os.getpid(), signal.SIGINT)

        return Starlette(
            debug=debug,
            routes=[
                Route("/health", endpoint=handle_health, methods=["GET"]),
                Route("/terminate", endpoint=handle_terminate, methods=["GET"]),
                Route("/sse", endpoint=handle_sse),
                Route("/messages", endpoint=handle_messages, methods=["POST"]),
            ],
        )

    async def run_stdio(self) -> None:
        """Run MCP server over stdin/stdout."""
        async with mcp.server.stdio.stdio_server() as (read_stream, write_stream):
            await self._server.run(
                read_stream,
                write_stream,
                self._create_initialization_options(),
            )

    @staticmethod
    def service_classes() -> list[type["MCPBaseService"]]:
        """
        Get all available MCP service classes.

        Returns:
            list[type[MCPBaseService]]: List of service class types

        """
        return locate_subclasses(MCPBaseService)  # type: ignore

    @staticmethod
    def tools() -> list[types.Tool]:
        """
        Get all available MCP tools.

        Returns:
            list[types.Tool]: List of available tools

        """
        return asyncio.run(MCPServer().tool_list())

    @staticmethod
    def tool(
        name: str,
        arguments: dict | None = None,
    ) -> list[types.TextContent | types.ImageContent | types.EmbeddedResource]:
        """
        Call an MCP tool by name.

        Args:
            name: Name of the tool to call
            arguments: Optional arguments to pass to the tool

        Returns:
            list[types.TextContent | types.ImageContent | types.EmbeddedResource]: Tool execution results

        """
        return asyncio.run(MCPServer().tool_call(name, arguments))

    @staticmethod
    def resources() -> list[types.Resource]:
        """
        Get all available MCP resources.

        Returns:
            list[types.Resource]: List of available resources

        """
        return asyncio.run(MCPServer().resource_list())

    @staticmethod
    def resource(uri: str) -> str:
        """
        Get a resource by URI.

        Args:
            uri: URI of the resource to get

        Returns:
            str: Resource content

        """
        return asyncio.run(MCPServer().resource_get(AnyUrl(uri)))

    @staticmethod
    def prompts() -> list[types.Prompt]:
        """
        Get all available prompts.

        Returns:
            list[types.Prompt]: List of available prompts

        """
        return asyncio.run(MCPServer().prompt_list())

    @staticmethod
    def prompt(name: str, arguments: dict | None = None) -> types.GetPromptResult:
        """
        Get a prompt by name.

        Args:
            name: Name of the prompt to get
            arguments: Optional arguments to pass to the prompt

        Returns:
            types.GetPromptResult: Prompt result

        """
        return asyncio.run(MCPServer().prompt_get(name, arguments))

    @staticmethod
    def serve(host: str | None = None, port: int | None = None, debug: bool = True) -> None:
        """
        Start the MCP server.

        Args:
            host: Host to bind to (for HTTP server)
            port: Port to bind to (for HTTP server)
            debug: Enable debug mode

        Returns:
            None

        """
        if host and port:
            import uvicorn  # noqa: PLC0415 (performance)

            uvicorn.run(
                MCPServer().starlette_app(debug),
                host=host,
                port=port,
                log_level=str.lower(os.environ.get("LOGLEVEL", "INFO")),
                log_config=None,
            )
        else:
            return asyncio.run(MCPServer().run_stdio())
        return None

    def _create_initialization_options(self) -> InitializationOptions:
        return InitializationOptions(
            server_name=__project_name__,
            server_version=__version__,
            capabilities=self._server.get_capabilities(
                notification_options=NotificationOptions(
                    prompts_changed=False,
                    resources_changed=False,
                    tools_changed=False,
                ),
                experimental_capabilities={},
            ),
        )

    @staticmethod
    def _marshal_result(  # noqa: PLR0911
        result: type[Any],
    ) -> list[TextContent | ImageContent | EmbeddedResource]:
        """
        Marshals a result into a sequence of TextContent, ImageContent, or EmbeddedResource.

        Returns:
            list[TextContent | ImageContent | EmbeddedResource]: The marshalled result sequence.

        """
        if result is None:
            return []

        if isinstance(result, TextContent | ImageContent | EmbeddedResource):
            return [result]

        if isinstance(result, str):
            return [TextContent(type="text", text=result)]

        if _has_imaging and isinstance(result, Image.Image):
            mime_type = (
                Image.MIME.get(result.format, "application/octet-stream")
                if result.format
                else "application/octet-stream"
            )
            data = BytesIO()
            result.save(data, format=result.format)
            return [
                ImageContent(
                    type="image",
                    data=base64.b64encode(data.getvalue()).decode("utf-8"),
                    mimeType=mime_type,
                ),
            ]

        if isinstance(result, list | tuple):
            return [item for subresult in result for item in MCPServer._marshal_result(subresult)]

        try:
            return [
                TextContent(
                    type="text",
                    text=json.dumps(pydantic_core.to_jsonable_python(result), indent=2),
                ),
            ]
        except Exception:
            logger.exception("Error converting result to JSON")

        return [TextContent(type="text", text=str(result))]



"""Tests for MCP CLI functionality."""

import json
import os
import subprocess
from pathlib import Path
from unittest.mock import patch

import pytest
from typer.testing import CliRunner

from starbridge.cli import cli

MOCK_GET_ALL_SPACES = "atlassian.Confluence.get_all_spaces"
MOCK_GET_SPACE = "atlassian.Confluence.get_space"
PYPROJECT_TOML = "pyproject.toml"
DOT_COVERAGE = ".coverage"


@pytest.fixture
def runner() -> CliRunner:
    """Get a Click CLI test runner."""
    return CliRunner()


def test_mcp_cli_services(runner) -> None:
    """Check available services."""
    result = runner.invoke(cli, ["mcp", "services"])
    assert result.exit_code == 0

    # We expect these three services to be present
    expected_services = [
        "starbridge.claude.service.Service",
        "starbridge.confluence.service.Service",
        "starbridge.hello.service.Service",
    ]

    for service in expected_services:
        assert service in result.stdout


def test_mcp_cli_tools(runner) -> None:
    """Check available tools."""
    result = runner.invoke(cli, ["mcp", "tools"])
    assert result.exit_code == 0

    # All expected tool names
    expected_tools = [
        "starbridge_claude_health",
        "starbridge_claude_info",
        "starbridge_claude_restart",
        "starbridge_confluence_health",
        "starbridge_confluence_info",
        "starbridge_confluence_page_create",
        "starbridge_confluence_page_delete",
        "starbridge_confluence_page_get",
        "starbridge_confluence_page_list",
        "starbridge_confluence_page_update",
        "starbridge_confluence_space_list",
        "starbridge_hello_bridge",
        "starbridge_hello_health",
        "starbridge_hello_hello",
        "starbridge_hello_info",
        "starbridge_hello_pdf",
    ]

    output = result.stdout
    for tool in expected_tools:
        assert f"name='{tool}'" in output


def test_mcp_cli_tool(runner) -> None:
    """Check a tool."""
    result = runner.invoke(
        cli,
        ["mcp", "tool", "starbridge_hello_hello", "--arguments", "locale=de_DE"],
    )
    assert result.exit_code == 0
    assert "Hallo Welt!" in result.stdout


def test_mcp_cli_prompts(runner) -> None:
    """Check available tools."""
    result = runner.invoke(cli, ["mcp", "prompts"])
    assert result.exit_code == 0
    assert "starbridge_confluence_space_summary" in result.stdout


@patch(MOCK_GET_ALL_SPACES)
def test_mcp_cli_prompt(mock_get_all_spaces, runner) -> None:
    """Check available resources."""
    # Mock the response data that would come from get_all_spaces
    with Path("tests/fixtures/get_all_spaces.json").open(encoding="utf-8") as f:
        mock_get_all_spaces.return_value = json.loads(f.read())

    result = runner.invoke(
        cli,
        [
            "mcp",
            "prompt",
            "starbridge_confluence_space_summary",
            "--arguments",
            "style=detailed",
        ],
    )

    assert result.exit_code == 0
    assert "details" in result.stdout


def test_mcp_cli_resource_types(runner) -> None:
    """Check available resources."""
    result = runner.invoke(cli, ["mcp", "resource-types"])
    assert result.exit_code == 0
    assert "starbridge://confluence/space" in result.stdout


@patch(MOCK_GET_ALL_SPACES)
def test_mcp_cli_resources(mock_get_all_spaces, runner) -> None:
    """Check available resources."""
    # Mock the response data that would come from get_all_spaces
    with Path("tests/fixtures/get_all_spaces.json").open(encoding="utf-8") as f:
        mock_get_all_spaces.return_value = json.loads(f.read())

    result = runner.invoke(cli, ["mcp", "resources"])
    assert result.exit_code == 0
    assert "7120201709026d2b41448e93bb58d" in result.stdout  # pragma: allowlist secret


@patch(MOCK_GET_SPACE)
def test_mcp_cli_resource(mock_get_space, runner) -> None:
    """Read a resource."""
    # Mock the response data that would come from get_all_spaces
    with Path("tests/fixtures/get_space.json").open(encoding="utf-8") as f:
        mock_get_space.return_value = json.loads(f.read())

    result = runner.invoke(
        cli,
        [
            "mcp",
            "resource",
            "starbridge://confluence/space/~7120201709026d2b41448e93bb58d5fa301026",  # pragma: allowlist secret
        ],
    )
    assert result.exit_code == 0
    assert "7120201709026d2b41448e93bb58d" in result.stdout  # pragma: allowlist secret


def test_mcp_cli_inspector(runner) -> None:
    """Test the MCP inspector functionality."""
    env = os.environ.copy()
    env.update({
        "COVERAGE_PROCESS_START": PYPROJECT_TOML,
        "COVERAGE_FILE": os.getenv("COVERAGE_FILE", DOT_COVERAGE),
        "MOCKS": "webbrowser.open",
        "MOCK_WEBBROWSER_OPEN": "return_value=None",
        "CLIENT_PORT": "5174",
        "SERVER_PORT": "3001",
    })

    process = None
    try:
        process = subprocess.run(
            ["uv", "run", "starbridge", "mcp", "inspect"],
            capture_output=True,
            timeout=10,
            text=True,
            env=env,
            check=False,
        )
    except subprocess.TimeoutExpired as e:
        process = e.stdout

    # Handle both cases where process completed or timed out
    if isinstance(process, subprocess.CompletedProcess):
        assert "Opened browser pointing to MCP Inspector" in process.stdout
    else:
        assert process is not None, "Process failed to execute"



"""CLI to interact with Confluence."""

import os
import re
import subprocess
import webbrowser
from typing import Annotated

import typer

from starbridge import __project_name__, __version__
from starbridge.utils import console, get_process_info

from .server import MCPServer

cli = typer.Typer(name="mcp", help="MCP operations")


@cli.command()
def health() -> None:
    """Check health of the services and their dependencies."""
    console.print_json(MCPServer().health().model_dump_json())


@cli.command()
def services() -> None:
    """Services exposed by modules."""
    console.print(MCPServer.service_classes())


@cli.command()
def tools() -> None:
    """Tools exposed by modules."""
    console.print(MCPServer.tools())


@cli.command()
def tool(
    name: str,
    arguments: Annotated[
        list[str] | None,
        typer.Option(help="Arguments in key=value format"),
    ] = None,
) -> None:
    """
    Get tool by name with optional arguments.

    Args:
        name (str): Name of the tool
        arguments (list[str]): Arguments in key=value format

    """
    args = {}
    if arguments:
        for arg in arguments:
            key, value = arg.split("=", 1)
            args[key] = value
    console.print(MCPServer.tool(name, args))


@cli.command()
def resources() -> None:
    """Resources exposed by modules."""
    console.print(MCPServer.resources())


@cli.command()
def resource(uri: str) -> None:
    """
    Get resource by URI.

    Args:
        uri (str): URI of the resources

    """
    console.print(MCPServer.resource(uri))


@cli.command()
def prompts() -> None:
    """Prompts exposed by modules."""
    console.print(MCPServer.prompts())


@cli.command()
def prompt(
    name: str,
    arguments: Annotated[
        list[str] | None,
        typer.Option(help="Arguments in key=value format"),
    ] = None,
) -> None:
    """
    Get a prompt by name with optional arguments.

    Args:
        name (str): Name of the prompt
        arguments (list[str]): Arguments in key=value format

    """
    args = {}
    if arguments:
        for arg in arguments:
            key, value = arg.split("=", 1)
            args[key] = value
    console.print(MCPServer.prompt(name, args))


@cli.command()
def resource_types() -> None:
    """Resource types exposed by modules."""
    console.print(MCPServer.resource_types())


@cli.command()
def serve(
    host: Annotated[
        str | None,
        typer.Option(
            help="Host to run the server on",
        ),
    ] = None,
    port: Annotated[
        int | None,
        typer.Option(
            help="Port to run the server on",
        ),
    ] = None,
    debug: Annotated[
        bool,
        typer.Option(
            help="Debug mode",
        ),
    ] = True,
    env: Annotated[  # Parsed in bootstrap.py  # noqa: ARG001
        list[str] | None,
        typer.Option(
            "--env",
            help="Environment variables in key=value format. Can be used multiple times in one call. "
            "Only STARBRIDGE_ prefixed vars are used. Example --env "
            'STARBRIDGE_ATLASSIAN_URL="https://your-domain.atlassian.net" '
            '--env STARBRIDGE_ATLASSIAN_EMAIL="YOUR_EMAIL"',
        ),
    ] = None,
) -> None:
    """
    Run MCP server.

    Args:
        host (str): Host to run the server on
        port (int): Port to run the server on
        debug (bool): Debug mode
        env (list[str]): Environment variables in key=value format. Can be used multiple times in one call.
            Only `STARBRIDGE_` prefixed vars are used. Example --env
            `STARBRIDGE_ATLASSIAN_URL="https://your-domain.atlassian.net" --env STARBRIDGE_ATLASSIAN_EMAIL="YOUR_EMAIL"`

    """
    MCPServer().serve(host, port, debug)


@cli.command()
def inspect() -> None:
    """Run inspector."""
    process_info = get_process_info()
    console.print(
        f"⭐ Starbridge controller: v{__version__} "
        f"(project root {process_info.project_root}, pid {process_info.pid}), "
        f"parent '{process_info.parent.name}' (pid {process_info.parent.pid})",
    )
    env_args = []
    for key, value in os.environ.items():
        if key.startswith("STARBRIDGE_"):
            env_args.extend(["--env", f'{key}="{value}"'])
    cmd = [
        "npx",
        "@modelcontextprotocol/inspector",
        "uv",
        "--directory",
        process_info.project_root,
        "run",
        "--no-dev",
        __project_name__,
        *env_args,
    ]
    console.print(f"Executing: {' '.join(cmd)}")

    process = subprocess.Popen(  # noqa: S603
        cmd,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
        text=True,
        env=os.environ,
    )

    url_pattern = r"MCP Inspector is up and running at (http://[^\s]+)"

    while True:
        line = process.stdout.readline() if process.stdout is not None else ""
        if not line:
            break
        match = re.search(url_pattern, line)
        if match:
            url = match.group(1)
            console.print(f"Opened browser pointing to MCP Inspector at {url}")
            webbrowser.open(url)

    process.wait()



"""Handles search interactions."""

import asyncio

from brave_search_python_client import (
    BraveSearch,
    WebSearchApiResponse,
    WebSearchRequest,
)

from starbridge.mcp import MCPBaseService, MCPContext, mcp_tool
from starbridge.utils import Health, get_logger

from .settings import Settings

logger = get_logger(__name__)


class Service(MCPBaseService):
    """Service class for search operations."""

    _settings: Settings
    _bs: BraveSearch

    def __init__(self) -> None:
        """Initialize the search service with Brave Search client."""
        super().__init__(Settings)
        self._bs = BraveSearch(api_key=self._settings.brave_search_api_key)

    @mcp_tool()
    def health(self, context: MCPContext | None = None) -> Health:  # noqa: ARG002
        """
        Check health of the search service.

        Args:
            context (MCPContext | None): MCP context for the operation

        Returns:
            Health: The health status of the search service.

        """
        if not asyncio.run(self._bs.is_connected()):
            return Health(
                status=Health.Status.DOWN,
                reason="Brave Search API not connected",
            )
        return Health(status=Health.Status.UP)

    @mcp_tool()
    def info(self, context: MCPContext | None = None) -> dict:  # noqa: ARG002, PLR6301
        """
        Info about search environment.

        Args:
            context (MCPContext | None): MCP context for the

        Returns:
            dict: Information about the search environment configuration.

        """
        return {}

    @mcp_tool()
    async def web(  # noqa: D417
        self,
        q: str,
        context: MCPContext | None = None,  # noqa: ARG002
    ) -> WebSearchApiResponse:
        """
        Search the world wide web via Brave Search.

        Should be called by the assistant when the user asks to search the Internet / the world wide web
            - This includes the case when the user asks to find something in the web

        Args:
            q (str): The query to search for

        Returns:
            (WebSearchApiResponse): JSON response containing search results returned by Brave Search API

        Raises:
            (BraveSearchClientError): If the request was formulated wrong
            (BraveSearchApiError): If the Brave Search API was not reachable or had an internal server error

        """
        return await self._bs.web(WebSearchRequest(q=q))



"""Test web MCP functionality."""

import pytest
from mcp import ClientSession
from mcp.client.stdio import stdio_client
from mcp.types import (
    TextContent,
)
from typer.testing import CliRunner

from tests.utils_test import _server_parameters

GET_TEST_URL = "https://starbridge.readthedocs.io/en/latest/"

PYPROJECT_TOML = "pyproject.toml"
DOT_COVERAGE = ".coverage"


@pytest.fixture
def runner() -> CliRunner:
    """Get a Click CLI test runner."""
    return CliRunner()


@pytest.mark.asyncio
async def test_web_mcp_tool_get() -> None:
    """Test server tool get."""
    async with stdio_client(_server_parameters()) as (read, write), ClientSession(read, write) as session:
        await session.initialize()

        result = await session.call_tool(
            "starbridge_web_get",
            {
                "url": GET_TEST_URL,
                "transform_to_markdown": True,
                "extract_links": False,
                "additional_context": False,
            },
        )
        assert len(result.content) == 1
        content = result.content[0]
        assert type(content) is TextContent
        assert "starbridge" in content.text



"""⭐ Integrates Claude Desktop with the web, Google and Atlassian workspaces."""

import importlib.metadata
import os
import pathlib
import sys

__project_name__ = __name__.split(".")[0]
__project_path__ = str(pathlib.Path(__file__).parent.parent.parent)
__version__ = importlib.metadata.version(__project_name__)
__is_development_mode__ = "uvx" not in sys.argv[0].lower()
__is_running_in_container__ = os.getenv("STARBRIDGE_RUNNING_IN_CONTAINER") is not None


def _parse_env_args() -> None:
    """Parse --env arguments from command line and add to environment if STARBRIDGE_ prefixed."""
    args = sys.argv[1:]
    i = 0
    while i < len(args):
        if (args[i] == "--env" or args[i] == "-e") and i + 1 < len(args):
            try:
                key, value = args[i + 1].split("=", 1)
                if key.startswith("STARBRIDGE_"):
                    # Strip quotes if present
                    value = value.strip("\"'")
                    os.environ[key] = value
            except ValueError:
                pass  # Silently skip malformed env vars
        i += 1


def _amend_library_path() -> None:
    """Patch environment variables before any other imports."""
    if "DYLD_FALLBACK_LIBRARY_PATH" not in os.environ:
        os.environ["DYLD_FALLBACK_LIBRARY_PATH"] = f"{os.getenv('HOMEBREW_PREFIX', '/opt/homebrew')}/lib/"


def _log_boot_message() -> None:
    # Local import as this initializes logging and instrumentation
    # which might depend on environment arguments parsed from argv
    from starbridge.utils import get_logger, get_process_info  # noqa: PLC0415

    logger = get_logger(__name__)

    process_info = get_process_info()
    logger.debug(
        "⭐ Booting Starbridge v%s (project root %s, pid %s), parent '%s' (pid %s)",
        __version__,
        process_info.project_root,
        process_info.pid,
        process_info.parent.name,
        process_info.parent.pid,
    )


# boot

_amend_library_path()
_parse_env_args()
_log_boot_message()

# exports

__all__ = [
    "__is_development_mode__",
    "__project_name__",
    "__project_path__",
    "__version__",
]



"""Test docker functionality of the Starbridge core package."""

import pytest
import requests
from requests.exceptions import ConnectionError as RequestsConnectionError


def is_responsive(url) -> bool | None:
    """
    Check if a given URL is responsive.

    Args:
        url: The URL to check

    Returns:
        bool: True if URL is responsive, False if not responsive
        None: If connection failed

    """
    try:
        response = requests.get(url, timeout=5.0)
        if response.status_code == 200:
            return True
    except RequestsConnectionError:
        return False


@pytest.fixture(scope="session")
def inspector_service(docker_ip, docker_services) -> str:
    """Ensure that HTTP service is up and responsive."""
    # `port_for` takes a container port and returns the corresponding host port
    port = docker_services.port_for("starbridge_inspector", 5173)
    url = f"http://{docker_ip}:{port}"
    docker_services.wait_until_responsive(
        timeout=5.0,
        pause=0.1,
        check=lambda: is_responsive(url),
    )
    return url


@pytest.mark.xdist_group(name="docker")
@pytest.mark.skip_with_act
def test_core_docker_inspector_healthy(inspector_service) -> None:
    """Test that the docker inspector service is healthy and responding."""
    status = 200
    response = requests.get(inspector_service + "/", timeout=5.0)

    assert response.status_code == status
    assert "mcp.svg" in response.content.decode("utf-8")


@pytest.mark.xdist_group(name="docker")
@pytest.mark.skip_with_act
def test_core_docker_cli_help_with_love(docker_services) -> None:
    """Test the CLI help command with docker services returns expected output."""
    out = docker_services._docker_compose.execute("run starbridge --help ")
    out_str = out.decode("utf-8")
    assert "built with love in Berlin" in out_str


@pytest.mark.xdist_group(name="docker")
@pytest.mark.skip_with_act
def test_core_docker_cli_mcp_services(docker_services) -> None:
    """Test the MCP services command with docker services returns expected output."""
    out = docker_services._docker_compose.execute("run starbridge mcp services")
    out_str = out.decode("utf-8")
    assert "claude" in out_str
    assert "confluence" in out_str
    assert "hello" in out_str



"""Tests for the Claude CLI functionality."""

import json
import os
import subprocess
from pathlib import Path
from unittest.mock import Mock, patch

import pytest
from typer.testing import CliRunner

from starbridge.cli import cli

SUBPROCESS_RUN = "subprocess.run"


@pytest.fixture
def runner() -> CliRunner:
    """Get a Click CLI test runner."""
    return CliRunner()


@patch("platform.system", return_value="Darwin")
@patch("psutil.process_iter")
@patch("starbridge.claude.service.Service.is_installed", return_value=True)
def test_claude_cli_health(mock_has_config, mock_process_iter, mock_platform, runner) -> None:
    """Check health spots Claude up and running."""
    mock_process = Mock()
    mock_process.info = {"pid": 1234, "name": "Claude"}
    mock_process_iter.return_value = [mock_process]

    result = runner.invoke(cli, ["claude", "health"])
    assert '"UP"' in result.stdout
    assert result.exit_code == 0


def test_claude_cli_health_in_container() -> None:
    """Check health spots Claude not running in container."""
    env = os.environ.copy()
    env.update({
        "COVERAGE_PROCESS_START": "pyproject.toml",
        "COVERAGE_FILE": os.getenv("COVERAGE_FILE", ".coverage"),
        "STARBRIDGE_RUNNING_IN_CONTAINER": "1",  # indicate we are running within a container
    })
    result = subprocess.run(
        ["uv", "run", "starbridge", "claude", "health"],
        capture_output=True,
        text=True,  # Get string output instead of bytes
        env=env,
        check=False,
    )
    assert result.returncode == 0
    assert '"DOWN"' in result.stdout


@patch("platform.system", return_value="Darwin")
@patch("psutil.process_iter")
@patch("starbridge.claude.service.Service.is_installed", return_value=True)
def test_claude_cli_info_works(
    mock_has_config,
    mock_process_iter,
    mock_platform,
    runner,
) -> None:
    """Check info spots spots installed and running Claude."""
    mock_process = Mock()
    mock_process.info = {
        "pid": 1234,
        "ppid": 0,
        "name": "Claude",
    }
    mock_process.cmdline = lambda: [
        "/Applications/Claude.app/Contents/MacOS/Claude",
        "--annotation=_productName=Claude",
    ]
    mock_process_iter.return_value = [mock_process]

    result = runner.invoke(cli, ["claude", "info"])
    assert result.exit_code == 0
    info = json.loads(result.stdout)
    assert info["is_installed"] is True
    assert info["is_running"] is True


def test_claude_cli_config_works(runner, tmp_path) -> None:
    """Check config works."""
    # Set up test config file in isolated tmp directory
    config_dir = tmp_path / "Claude"
    config_dir.mkdir()
    config_file = config_dir / "claude_desktop_config.json"
    test_config = {"mcpServers": {"starbridge": {"command": "uv"}}}
    config_file.write_text(json.dumps(test_config))

    with patch(
        "starbridge.claude.service.Service.application_directory",
        return_value=config_dir,
    ):
        result = runner.invoke(cli, ["claude", "config"])

        assert result.exit_code == 0
        assert '"mcpServers"' in result.stdout
        assert '"starbridge"' in result.stdout


@patch("starbridge.claude.service.Service.is_installed", return_value=False)
def test_claude_cli_config_fails_if_not_installed(mock_is_installed, runner) -> None:
    """Check config fails when config path is not a file."""
    result = runner.invoke(cli, ["claude", "config"])
    assert result.exit_code == 0
    assert "not installed" in result.stdout


@patch("starbridge.claude.service.Service.is_installed", return_value=True)
@patch(
    "starbridge.claude.service.Service.config_path",
    return_value=Path("/nonexistent/config.json"),
)
def test_claude_cli_config_fails_if_no_config(
    mock_is_installed,
    mock_config_path,
    runner,
) -> None:
    """Check config fails when no config file."""
    result = runner.invoke(cli, ["claude", "config"])
    assert result.exit_code == 0
    assert "No config" in result.stdout


def test_claude_cli_log_works(runner, tmp_path, capfd) -> None:
    """Check showing log works."""
    # Set up test config file in isolated tmp directory
    log_dir = tmp_path / "Claude"
    log_dir.mkdir()
    main_log_file = log_dir / "mcp.log"
    main_log_file.write_text("claude\n")
    starbridge_log_file = log_dir / "mcp-server-starbridge.log"
    starbridge_log_file.write_text("starbridge\n")

    with patch(
        "starbridge.claude.service.Service.log_directory",
        return_value=log_dir,
    ):
        result = runner.invoke(cli, ["claude", "log", "--name", "main"])
        assert result.exit_code == 0
        out, _ = capfd.readouterr()
        assert "claude" in out
        result = runner.invoke(cli, ["claude", "log"])
        assert result.exit_code == 0
        out, _ = capfd.readouterr()
        assert "starbridge" in out


@patch("starbridge.claude.service.Service.is_installed", return_value=False)
def test_claude_cli_restart_fails_if_not_installed(mock_is_installed, runner) -> None:
    """Check restart fails if Claude not installed."""
    result = runner.invoke(cli, ["claude", "restart"])
    assert result.exit_code == 0
    assert "not installed" in result.stdout


@patch("starbridge.claude.service.Service.is_installed", return_value=True)
@patch("psutil.process_iter")
@patch("subprocess.run", return_value=subprocess.CompletedProcess([], 0))
def test_claude_cli_restart_works_if_installed(
    mock_subprocess_run,
    mock_process_iter,
    mock_is_installed,
    runner,
) -> None:
    """Check restart works on supported platforms."""
    mock_process_claude = Mock()
    mock_process_claude.info = {"name": "Claude"}
    mock_process_other = Mock()
    mock_process_other.info = {"name": "Other"}
    mock_process_iter.return_value = [mock_process_other, mock_process_claude]

    platform_commands = {
        "Darwin": {"args": ["/usr/bin/open", "-a", "Claude"], "shell": False},
        "win23": {"args": ["start", "Claude"], "shell": True},
        "Linux": {"args": ["xdg-open", "Claude"], "shell": False},
    }

    for platform_name, command in platform_commands.items():
        with patch("platform.system", return_value=platform_name):
            result = runner.invoke(cli, ["claude", "restart"])
            mock_process_claude.terminate.assert_called_once()
            assert mock_process_other.terminate.call_count == 0
            mock_subprocess_run.assert_called_once_with(
                command["args"],
                shell=command["shell"],
                check=True,
            )
            assert result.exit_code == 0
            assert "was restarted" in result.stdout
            mock_process_claude.reset_mock()
            mock_subprocess_run.reset_mock()


@patch("starbridge.claude.service.Service.is_installed", return_value=True)
@patch("psutil.process_iter")
@patch("subprocess.run", return_value=subprocess.CompletedProcess([], 0))
def test_claude_cli_restart_fails_on_unknown_system(
    mock_subprocess_run,
    mock_process_iter,
    mock_is_installed,
    runner,
) -> None:
    """Check restart fails if Claude running on unknown platform."""
    mock_process = Mock()
    mock_process.info = {"name": "Claude"}
    mock_process_iter.return_value = [mock_process]

    with patch("platform.system", return_value="AmigaOS"):
        result = runner.invoke(cli, ["claude", "restart"])
        mock_process.terminate.assert_called_once()
        assert result.exit_code == 1
        assert mock_subprocess_run.called is False
        mock_process.reset_mock()
        mock_subprocess_run.reset_mock()


@patch("starbridge.claude.service.Service.is_installed", return_value=True)
@patch(
    "starbridge.claude.service.Service.platform_supports_restart",
    return_value=False,
)  # TODO(@helmut-hoffer-von-ankershoffen): find out why we cannot patch is_running_in_container
def test_claude_cli_restart_fails_in_container(
    mock_platform_supports_restart,
    mock_is_installed,
    runner,
) -> None:
    """Check restart fails if Claude running in container."""
    result = runner.invoke(cli, ["claude", "restart"])
    assert result.exit_code == 1


def test_claude_cli_in_container_commands_not_available() -> None:
    """Check log command not available in container."""
    env = os.environ.copy()
    env.update({
        "COVERAGE_PROCESS_START": "pyproject.toml",
        "COVERAGE_FILE": os.getenv("COVERAGE_FILE", ".coverage"),
        "STARBRIDGE_RUNNING_IN_CONTAINER": "1",  # indicate we are running within a container
    })
    # Test commands not available in container
    for command in ["log", "restart", "config"]:
        result = subprocess.run(
            ["uv", "run", "starbridge", "claude", command],
            capture_output=True,
            text=True,
            env=env,
            check=False,
        )
        assert result.returncode == 2
        assert "No such command" in result.stderr



"""Claude Desktop application integration module."""

from .cli import cli
from .service import Service
from .util import generate_mcp_server_config

__all__ = ["Service", "cli", "generate_mcp_server_config"]



"""Settings used for interacting with the world wide web."""

from typing import Annotated

from pydantic import Field
from pydantic_settings import BaseSettings, SettingsConfigDict

from starbridge import __project_name__, __version__


class Settings(BaseSettings):
    """Settings for web module."""

    model_config = SettingsConfigDict(
        env_prefix=f"{__project_name__.upper()}_WEB_",
        extra="ignore",
        env_file=".env",
        env_file_encoding="utf-8",
    )

    user_agent: Annotated[
        str,
        Field(
            default=f"starbridge/{__version__} (Autonomous; https://github.com/helmut-hoffer-von-ankershoffen/starbridge)",
            description="User agent to use when fetching URLs.",
        ),
    ]

    respect_robots_txt: Annotated[
        bool,
        Field(
            default=True,
            description="Whether to respect robots.txt files when interacting with the web as an agent.",
        ),
    ]

    timeout: Annotated[int, Field(default=60, description="Timeout for web requests.")]



"""Tests for Confluence CLI functionality."""

import json
from pathlib import Path
from unittest.mock import patch

import pytest
from typer.testing import CliRunner

from starbridge.cli import cli

MOCK_GET_ALL_SPACES = "atlassian.Confluence.get_all_spaces"
MOCK_GET_SPACE = "atlassian.Confluence.get_space"
MOCK_GET_ALL_PAGES_FROM_SPACE = "atlassian.Confluence.get_all_pages_from_space"
MOCK_GET_PAGE_BY_ID = "atlassian.Confluence.get_page_by_id"
MOCK_CQL = "atlassian.Confluence.cql"
FIXTURE_GET_ALL_SPACES = "tests/fixtures/get_all_spaces.json"


@pytest.fixture
def runner() -> CliRunner:
    """Get a Click CLI test runner."""
    return CliRunner()


@patch(MOCK_GET_ALL_SPACES)
def test_confluence_cli_health(mock_get_all_spaces, runner) -> None:
    """Check health."""
    # Mock the response data that would come from get_all_spaces
    with Path(FIXTURE_GET_ALL_SPACES).open(encoding="utf-8") as f:
        mock_get_all_spaces.return_value = json.loads(f.read())

    result = runner.invoke(cli, ["confluence", "health"])
    assert '"UP"' in result.output
    assert result.exit_code == 0


def test_confluence_cli_info(runner) -> None:
    """Check info."""
    result = runner.invoke(cli, ["confluence", "info"])
    assert result.exit_code == 0


@patch(MOCK_GET_ALL_SPACES)
def test_confluence_cli_mcp_resources(mock_get_all_spaces, runner) -> None:
    """Check fetching resources."""
    # Mock the response data that would come from get_all_spaces
    with Path(FIXTURE_GET_ALL_SPACES).open(encoding="utf-8") as f:
        mock_get_all_spaces.return_value = json.loads(f.read())

    result = runner.invoke(cli, ["confluence", "mcp", "resources"])
    assert "helmut" in result.output
    assert result.exit_code == 0


def test_confluence_mcp_resource_types(runner) -> None:
    """Check resource types including space."""
    result = runner.invoke(cli, ["confluence", "mcp", "resource-types"])
    assert "space" in result.output
    assert result.exit_code == 0


@patch(MOCK_GET_SPACE)
def test_confluence_cli_mcp_space(mock_get_space, runner) -> None:
    """Check getting space."""
    # Mock the response data that would come from get_all_spaces
    with Path("tests/fixtures/get_space.json").open(encoding="utf-8") as f:
        mock_get_space.return_value = json.loads(f.read())

    result = runner.invoke(
        cli,
        ["confluence", "mcp", "space", "~7120201709026d2b41448e93bb58d5fa301026"],
    )
    assert "helmut" in result.output
    assert result.exit_code == 0


def test_confluence_cli_mcp_prompts(runner) -> None:
    """Check prompts."""
    result = runner.invoke(cli, ["confluence", "mcp", "prompts"])
    assert "summary" in result.output
    assert result.exit_code == 0


@patch(MOCK_GET_ALL_SPACES)
def test_confluence_cli_mcp_space_summary(mock_get_all_spaces, runner) -> None:
    """Check space list."""
    # Mock the response data that would come from get_all_spaces
    with Path(FIXTURE_GET_ALL_SPACES).open(encoding="utf-8") as f:
        mock_get_all_spaces.return_value = json.loads(f.read())

    result = runner.invoke(
        cli,
        ["confluence", "mcp", "space-summary", "--style", "detailed"],
    )
    assert "helmut" in result.output
    assert result.exit_code == 0


def test_confluence_cli_mcp_tools(runner) -> None:
    """Check tools include listing spaces and creating pages."""
    result = runner.invoke(cli, ["confluence", "mcp", "tools"])
    assert result.exit_code == 0
    assert "name='starbridge_confluence_info'" in result.stdout
    assert "name='starbridge_confluence_page_create'" in result.stdout
    assert "name='starbridge_confluence_space_list'" in result.stdout


@patch(MOCK_GET_ALL_SPACES)
def test_confluence_space_list(mock_get_all_spaces, runner) -> None:
    """Check space list."""
    # Mock the response data that would come from get_all_spaces
    with Path(FIXTURE_GET_ALL_SPACES).open(encoding="utf-8") as f:
        mock_get_all_spaces.return_value = json.loads(f.read())

    result = runner.invoke(cli, ["confluence", "space", "list"])
    assert "helmut" in result.output
    assert result.exit_code == 0


@patch(MOCK_GET_PAGE_BY_ID)
def test_confluence_cli_page_read(get_page_by_id, runner) -> None:
    """Check page list."""
    # Mock the response data that would come from get_all_spaces
    with Path("tests/fixtures/get_page_by_id.json").open(encoding="utf-8") as f:
        get_page_by_id.return_value = json.loads(f.read())
    result = runner.invoke(
        cli,
        [
            "confluence",
            "page",
            "read",
            "--page-id",
            '"2088927594"',
        ],
    )
    assert "Amazon Leadership Principles" in result.output
    assert result.exit_code == 0


@patch(MOCK_CQL)
def test_confluence_cli_page_search(cql, runner) -> None:
    """Check page list."""
    # Mock the response data that would come from get_all_spaces
    with Path("tests/fixtures/confluence_page_search.json").open(encoding="utf-8") as f:
        cql.return_value = json.loads(f.read())
    result = runner.invoke(
        cli,
        [
            "confluence",
            "page",
            "search",
            "--query",
            '"title ~ Helmut"',
        ],
    )
    assert "4711" in result.output
    assert result.exit_code == 0


@patch(MOCK_GET_ALL_PAGES_FROM_SPACE)
def test_confluence_cli_page_list(mock_get_all_pages_from_space, runner) -> None:
    """Check page list."""
    # Mock the response data that would come from get_all_spaces
    with Path("tests/fixtures/get_all_pages_from_space.json").open(encoding="utf-8") as f:
        mock_get_all_pages_from_space.return_value = json.loads(f.read())
    result = runner.invoke(
        cli,
        [
            "confluence",
            "page",
            "list",
            "--space-key",
            '"~7120201709026d2b41448e93bb58d5fa301026"',
        ],
    )
    assert "Amazon Leadership Principles" in result.output
    assert result.exit_code == 0



"""Tests for the web service functionality."""

import asyncio
from unittest.mock import patch

import pytest

from starbridge.web import RobotForbiddenError, Service

GET_TEST_TEXT_URL = (
    "https://github.com/helmut-hoffer-von-ankershoffen/starbridge/raw/refs/heads/main/tests/fixtures/starbridge.txt"
)
GET_TEST_MARKDOWN_URL = (
    "https://github.com/helmut-hoffer-von-ankershoffen/starbridge/raw/refs/heads/main/tests/fixtures/starbridge.md"
)
GET_TEST_PDF_URL = (
    "https://github.com/helmut-hoffer-von-ankershoffen/starbridge/raw/refs/heads/main/tests/fixtures/starbridge.pdf"
)
GET_TEST_WORD_URL = (
    "https://github.com/helmut-hoffer-von-ankershoffen/starbridge/raw/refs/heads/main/tests/fixtures/starbridge.docx"
)
GET_TEST_EXCEL_URL = (
    "https://github.com/helmut-hoffer-von-ankershoffen/starbridge/raw/refs/heads/main/tests/fixtures/starbridge.xlsx"
)


def test_web_service_get_forbidden() -> None:
    """Check getting content from the web fails if forbidden by robots.txt."""
    from starbridge.web import Service

    with pytest.raises(RobotForbiddenError):
        asyncio.run(Service().get("https://github.com/search/advanced"))


@patch(
    "starbridge.web.utils._ensure_allowed_to_crawl",
    return_value=None,
)
@pytest.mark.asyncio
async def test_web_service_get_pdf(runner) -> None:
    """Check getting content from the web as markdown."""
    result = await Service().get(
        url=GET_TEST_PDF_URL,
    )
    assert "Hello World" in (result.resource.text or "")


@patch(
    "starbridge.web.utils._ensure_allowed_to_crawl",
    return_value=None,
)
@pytest.mark.asyncio
async def test_web_service_get_word(runner) -> None:
    """Check getting content from the web as markdown."""
    result = await Service().get(
        url=GET_TEST_WORD_URL,
    )
    assert "# Headline" in (result.resource.text or "")


@patch(
    "starbridge.web.utils._ensure_allowed_to_crawl",
    return_value=None,
)
@pytest.mark.asyncio
async def test_web_service_get_excel(runner) -> None:
    """Check getting content from the web as markdown."""
    result = await Service().get(
        url=GET_TEST_EXCEL_URL,
    )
    assert "Starbridge" in (result.resource.text or "")


@patch(
    "starbridge.web.utils._ensure_allowed_to_crawl",
    return_value=None,
)
@pytest.mark.asyncio
async def test_web_service_get_text(runner) -> None:
    """Check getting content from the web as markdown."""
    result = await Service().get(
        url=GET_TEST_TEXT_URL,
    )
    assert "Lorem Ipsum" in (result.resource.text or "")


@patch(
    "starbridge.web.utils._ensure_allowed_to_crawl",
    return_value=None,
)
@pytest.mark.asyncio
async def test_web_service_get_markdown(runner) -> None:
    """Check getting content from the web as markdown."""
    result = await Service().get(
        url=GET_TEST_MARKDOWN_URL,
    )
    assert "Lorem Ipsum" in (result.resource.text or "")



"""Tests for core CLI functionality."""

import json
import os
import subprocess
import sys
from pathlib import Path
from unittest.mock import patch

import pytest
from typer.testing import CliRunner

from starbridge import __project_name__, __version__
from starbridge.cli import cli

INSTALLATION_INPUT = (
    "https://test.atlassian.net\n"
    "test@test.com\n"  # Atlassian email address
    "TEST_CONFLUENCE_API_TOKEN\n"  # Atlassian token
    "TEST_LOGFIRE_TOKEN\n"  # logfire token
    "production\n"  # logfire environment
    "1\n"  # instrument mcp server
    "0\n"  # instrument system metrics
    "TEST\n"  # brave search api
    "INFO\n"  # log level
    "1\n"  # log to file
    "test.log\n"  # log file
    "0\n"  # log to console
    "TEST_USER_AGENT\n"  # custom user agent
    "1\n"  # respect robots.txt
    "5\n"  # http timeout
)


@pytest.fixture
def runner() -> CliRunner:
    """Get a Click CLI test runner."""
    return CliRunner()


def test_core_cli_built_with_love(runner) -> None:
    """Check epilog shown."""
    result = runner.invoke(cli, ["--help"])
    assert result.exit_code == 0
    assert "built with love in Berlin" in result.output


def test_core_cli_invalid_command(runner) -> None:
    """Test invalid command returns error."""
    result = runner.invoke(cli, ["invalid"])
    assert result.exit_code != 0


def test_core_cli_info(runner) -> None:
    """Check processes exposed and version matching."""
    result = runner.invoke(cli, ["info"])
    assert result.exit_code == 0
    data = json.loads(result.stdout)
    assert data["name"] == __project_name__
    assert data["version"] == __version__


def test_core_cli_create_dot_env(runner, tmp_path) -> None:
    """Check configuration injected in claude as expected."""
    with runner.isolated_filesystem():
        inputs = INSTALLATION_INPUT
        result = runner.invoke(cli, ["create-dot-env"], input=inputs)
        assert result.exit_code == 0
        dot_env = Path(".env").read_text(encoding="utf-8")
        assert "STARBRIDGE_ATLASSIAN_URL=https://test.atlassian.net" in dot_env
        assert "STARBRIDGE_ATLASSIAN_EMAIL_ADDRESS=test@test.com" in dot_env
        assert "STARBRIDGE_ATLASSIAN_API_TOKEN=TEST_CONFLUENCE_API_TOKEN" in dot_env
        assert "STARBRIDGE_LOGFIRE_TOKEN=TEST_LOGFIRE_TOKEN" in dot_env
        assert "STARBRIDGE_LOGFIRE_ENVIRONMENT=production" in dot_env
        assert "STARBRIDGE_LOGFIRE_INSTRUMENT_MCP=1" in dot_env
        assert "STARBRIDGE_LOGFIRE_INSTRUMENT_SYSTEM_METRICS=0" in dot_env
        assert "STARBRIDGE_SEARCH_BRAVE_SEARCH_API_KEY=TEST" in dot_env
        assert "STARBRIDGE_LOGGING_LOGLEVEL=INFO" in dot_env
        assert "STARBRIDGE_LOGGING_LOG_FILE_ENABLED=1" in dot_env
        assert "STARBRIDGE_LOGGING_LOG_FILE_NAME=test.log" in dot_env
        assert "STARBRIDGE_LOGGING_LOG_CONSOLE_ENABLED=0" in dot_env
        assert "STARBRIDGE_WEB_USER_AGENT=TEST_USER_AGENT" in dot_env
        assert "STARBRIDGE_WEB_RESPECT_ROBOTS_TXT=1" in dot_env
        assert "STARBRIDGE_WEB_TIMEOUT=5" in dot_env


def test_core_cli_install(runner, tmp_path) -> None:  # noqa: PLR0915
    """Check configuration injected in claude as expected."""
    with patch(
        "starbridge.claude.service.Service.application_directory",
        return_value=tmp_path,
    ):
        mock_restart = patch(
            "starbridge.claude.service.Service._restart",
            return_value=None,
        ).start()
        inputs = INSTALLATION_INPUT
        result = runner.invoke(
            cli,
            ["install"],
            input=inputs,
        )
        assert result.exit_code == 0
        assert mock_restart.call_count == 1

        result = runner.invoke(cli, ["claude", "config"], input=inputs)
        assert result.exit_code == 0
        # Find start of JSON by looking for first '{'
        json_start = result.output.find("{")
        output_json = json.loads(result.output[json_start:])
        server_config = output_json["mcpServers"]["starbridge"]
        assert server_config["env"]["STARBRIDGE_ATLASSIAN_URL"] == "https://test.atlassian.net"
        assert server_config["env"]["STARBRIDGE_ATLASSIAN_EMAIL_ADDRESS"] == "test@test.com"
        assert server_config["env"]["STARBRIDGE_ATLASSIAN_API_TOKEN"] == "TEST_CONFLUENCE_API_TOKEN"  # noqa: S105

        mock_restart.reset_mock()
        result = runner.invoke(cli, ["uninstall"], input=inputs)
        assert mock_restart.call_count == 1
        assert result.exit_code == 0

        result = runner.invoke(cli, ["claude", "config"], input=inputs)
        assert result.exit_code == 0
        json_start = result.output.find("{")
        output_json = json.loads(result.output[json_start:])
        assert output_json["mcpServers"].get("starbridge") is None

        mock_restart.reset_mock()
        inputs = INSTALLATION_INPUT
        result = runner.invoke(cli, ["install", "--no-restart-claude"], input=inputs)
        assert mock_restart.call_count == 0
        assert result.exit_code == 0

        mock_restart.reset_mock()
        result = runner.invoke(cli, ["uninstall", "--no-restart-claude"], input=inputs)
        assert mock_restart.call_count == 0
        assert result.exit_code == 0

        mock_restart.reset_mock()
        result = runner.invoke(cli, ["uninstall"], input=inputs)
        assert mock_restart.call_count == 0
        assert result.exit_code == 0

        mock_restart.reset_mock()
        inputs = INSTALLATION_INPUT
        result = runner.invoke(cli, ["install"], input=inputs)
        assert mock_restart.call_count == 1
        assert result.exit_code == 0

        mock_restart.reset_mock()
        inputs = INSTALLATION_INPUT
        result = runner.invoke(cli, ["install"], input=inputs)
        assert mock_restart.call_count == 1
        assert result.exit_code == 0

        mock_restart.reset_mock()
        inputs = INSTALLATION_INPUT
        result = runner.invoke(cli, ["install", "--no-restart-claude"], input=inputs)
        assert mock_restart.call_count == 0
        assert result.exit_code == 0


def test_core_cli_main_guard() -> None:
    """Test the main CLI guard functionality."""
    env = os.environ.copy()
    env.update({
        "COVERAGE_PROCESS_START": "pyproject.toml",
        "COVERAGE_FILE": os.getenv("COVERAGE_FILE", ".coverage"),
    })
    result = subprocess.run(
        [sys.executable, "-m", "starbridge.cli", "hello", "hello"],
        capture_output=True,
        text=True,  # Get string output instead of bytes
        env=env,
        check=False,
    )
    assert result.returncode == 0
    assert "Hello World!" in result.stdout


def test_core_cli_main_guard_fail() -> None:
    """Test the main CLI guard failure cases."""
    env = os.environ.copy()
    env.update({
        "COVERAGE_PROCESS_START": "pyproject.toml",
        "COVERAGE_FILE": os.getenv("COVERAGE_FILE", ".coverage"),
        "MOCKS": "starbridge_hello_service_hello_fail",
    })
    result = subprocess.run(
        [sys.executable, "-m", "starbridge.cli", "hello", "hello"],
        capture_output=True,
        text=True,
        env=env,
        check=False,  # Get string output instead of bytes
    )
    assert result.returncode == 1
    assert "Fatal error occurred: Hello World failed" in result.stdout



"""Tests for core health CLI functionality."""

import json
from unittest.mock import patch

import pytest
from typer.testing import CliRunner

from starbridge.cli import cli
from starbridge.utils import Health


@pytest.fixture
def runner() -> CliRunner:
    """Get a Click CLI test runner."""
    return CliRunner()


@patch("starbridge.hello.service.Service.health")
def test_core_cli_health_down(mock_health, runner) -> None:
    """Check health output when a service is down."""
    mock_health.return_value = Health(status=Health.Status.DOWN, reason="testing")
    result = runner.invoke(cli, ["health", "--json"])
    assert result.exit_code == 0

    # Clean up any whitespace/newlines before parsing
    health_data = json.loads(result.output.replace("\n", ""))
    assert "dependencies" in health_data
    assert health_data["dependencies"]["hello"]["status"] == "DOWN"
    assert health_data["dependencies"]["hello"]["reason"] == "testing"


@patch("starbridge.claude.service.Service.is_installed")
@patch("starbridge.claude.service.Service.is_running")
@patch("starbridge.confluence.service.Service.space_list")
def test_core_cli_health_all_up(
    mock_space_list,
    mock_is_running,
    mock_is_installed,
    runner,
) -> None:
    """Check health when all services are up."""
    mock_is_installed.return_value = True
    mock_is_running.return_value = True
    mock_space_list.return_value = {"results": [{"name": "Test Space"}]}

    result = runner.invoke(cli, ["health", "--json"])
    assert result.exit_code == 0

    # Clean up any whitespace/newlines before parsing
    health_data = json.loads(result.output.replace("\n", ""))
    assert health_data["healthy"] is True
    assert "dependencies" in health_data

    # Verify each service status
    assert health_data["dependencies"]["claude"]["status"] == "UP"
    assert health_data["dependencies"]["confluence"]["status"] == "UP"
    assert health_data["dependencies"]["hello"]["status"] == "UP"

    # Verify no reasons for UP statuses
    for service in ["claude", "confluence", "hello"]:
        assert health_data["dependencies"][service].get("reason") is None



"""Sphinx configuration."""  # noqa: INP001

import re
from datetime import UTC, datetime

extensions = [
    "sphinx_toolbox.collapse",  # https://sphinx-toolbox.readthedocs.io/
    "sphinx_toolbox.sidebar_links",
    "sphinx_toolbox.github",
    "sphinx_toolbox.source",
    "sphinx.ext.autodoc",
    #    "enum_tools.autoenum",  # https://github.com/domdfcoding/enum_tools/tree/master  # noqa: ERA001
    "sphinx.ext.napoleon",  # https://sphinxcontrib-napoleon.readthedocs.io/en/latest/
    "sphinx-pydantic",
    "sphinxcontrib.autodoc_pydantic",  # https://autodoc-pydantic.readthedocs.io/en/stable/users/examples.html
    "sphinx.ext.coverage",
    "sphinx_copybutton",
    "sphinx.ext.extlinks",  # https://www.sphinx-doc.org/en/master/usage/extensions/extlinks.html
    "sphinx.ext.imgconverter",
    "sphinx_inline_tabs",
    "sphinx_mdinclude",
    "sphinxext.opengraph",
    "swagger_plugin_for_sphinx",  # https://github.com/SAP/swagger-plugin-for-sphinx?tab=readme-ov-file
    "sphinx_selective_exclude.eager_only",  # https://github.com/pfalcon/sphinx_selective_exclude?tab=readme-ov-file
    "sphinx_selective_exclude.search_auto_exclude",
    "sphinx_selective_exclude.modindex_exclude",
]

project = "starbridge"
author = "Helmut Hoffer von Ankershoffen"
copyright = f" (c) 2025-{datetime.now(UTC).year}, {author}"  # noqa: A001
version = "0.0.88"
release = version
github_username = "helmut-hoffer-von-ankershoffen"
github_repository = "starbridge"

language = "en"

ogp_site_name = "Starbridge"
ogp_image = "https://starbridge.readthedocs.io/en/latest/_static/logo.png"
ogp_custom_meta_tags = [
    '<meta name="twitter:card" content="Starbridge" />',
]
ogp_enable_meta_description = True
ogp_description_length = 300

autodoc_pydantic_model_show_json = False

napoleon_google_docstring = True
napoleon_numpy_docstring = False
napoleon_include_init_with_doc = False
napoleon_include_private_with_doc = False
napoleon_include_special_with_doc = True
napoleon_use_admonition_for_examples = True
napoleon_use_admonition_for_notes = True
napoleon_use_admonition_for_references = True
napoleon_use_ivar = False
napoleon_use_param = True
napoleon_use_rtype = True
napoleon_preprocess_types = True
napoleon_type_aliases = None
napoleon_attr_annotations = True

linkcheck_retries = 2
linkcheck_timeout = 1
linkcheck_workers = 10
linkcheck_ignore = [
    r"http://127\.0\.0\.1",
    r"http://localhost",
]


templates_path = ["_templates"]
exclude_patterns = []

html_theme = "furo"
html_static_path = ["_static"]
html_logo = "../../logo.png"
html_theme_options = {
    "announcement": (
        '<a target="_blank" href="https://github.com/helmut-hoffer-von-ankershoffen/starbridge">GitHub</a> - '
        '<a target="_blank" href="https://pypi.org/project/starbridge">PyPI</a> - '
        '<a target="_blank" href="https://hub.docker.com/r/helmuthva/starbridge/tags">Docker</a> - '
        '<a target="_blank" href="https://sonarcloud.io/summary/new_code?id=helmut-hoffer-von-ankershoffen_starbridge">SonarQube</a> - '  # noqa: E501
        '<a target="_blank" href="https://app.codecov.io/gh/helmut-hoffer-von-ankershoffen/starbridge">Codecov</a>'
    ),
}

latex_engine = "lualatex"  # https://github.com/readthedocs/readthedocs.org/issues/8382

# If true, show page references after internal links.
latex_show_pagerefs = True

# If true, show URL addresses after external links.
latex_show_urls = "footnote"

# If false, no module index is generated.
latex_domain_indices = True

# See https://www.sphinx-doc.org/en/master/latex.html
latex_elements = {
    # The paper size ('letterpaper' or 'a4paper').
    "papersize": "a4paper",
    # The font size ('10pt', '11pt' or '12pt').
    "pointsize": "10pt",
    # https://github.com/sphinx-doc/sphinx/issues/12332.
    "preamble": r"""
\directlua {
  luaotfload.add_fallback("emoji",
  {
     "[TwemojiMozilla.ttf]:mode=harf;",
     "[DejaVuSans.ttf]:mode=harf;",
  }
  )
}
\setmainfont{LatinModernRoman}[RawFeature={fallback=emoji},SmallCapsFont={* Caps}]
\setsansfont{LatinModernSans}[RawFeature={fallback=emoji}]
\setmonofont{DejaVuSansMono}[RawFeature={fallback=emoji},Scale=0.8]
    """,
}

slug = re.sub(r"\W+", "-", project.lower())

latex_documents = [
    ("index", f"{slug}.tex", rf"{project} Documentation", author, "manual", False),
]

latex_logo = "../../logo.png"



"""CLI to interact with Hello World."""

import os
import subprocess
import sys
import tempfile
import time
from pathlib import Path
from typing import Annotated

import typer
from rich.panel import Panel
from rich.text import Text

from starbridge.utils.console import console

from .service import Service

cli = typer.Typer(name="hello", help="Hello operations")


@cli.command()
def health() -> None:
    """Health of Hello World."""
    console.print_json(Service().health().model_dump_json())


@cli.command()
def info() -> None:
    """Info about Hello World."""
    console.print_json(data=Service().info())


@cli.command()
def hello(locale: Annotated[str, typer.Option(help="Locale to use")] = "en_US") -> None:
    """
    Print Hello World.

    Args:
        locale (str): Locale to use

    """
    console.print(Service().hello(locale))


if hasattr(Service, "bridge"):

    @cli.command()
    def bridge(
        dump: Annotated[
            bool,
            typer.Option(
                help="If set, will dump to file starbridge.png in current working directory. "
                "Defaults to opening viewer to show the image.",
            ),
        ] = False,
    ) -> None:
        """
        Show image of starbridge.

        Args:
            dump (bool): If set, will dump to file starbridge.png in current working directory.
                Defaults to opening viewer to show the image.

        """
        try:
            image = Service().bridge()
            if dump:
                image.save("starbridge.png")
            else:
                image.show()
        except OSError:
            text = Text()
            text.append("Please follow setup instructions for starbridge ")
            text.append("Install the library needed for image manipulation using:\n")
            text.append("• macOS: ", style="yellow")
            text.append("brew install cairo\n")
            text.append("• Linux: ", style="yellow")
            text.append("sudo apt-get install libcairo2")
            console.print(
                Panel(
                    text,
                    title="Setup Required: Cairo not found",
                    border_style="red",
                ),
            )
            sys.exit(78)


@cli.command()
def pdf(
    dump: Annotated[
        bool,
        typer.Option(
            help="If set, will dump to file starbridge.pdf in current working directory. "
            "Defaults to opening viewer to show the document.",
        ),
    ] = False,
) -> None:
    """
    Show pdf of starbridge.

    Args:
        dump (bool): If set, will dump to file starbridge.pdf in current working directory.
            Defaults to opening viewer to show the document.

    """
    pdf = Service().pdf_bytes()

    if dump:
        pdf_path = Path("starbridge.pdf")
        pdf_path.write_bytes(pdf)
    else:
        # Create temporary file that gets auto-deleted when closed
        with tempfile.NamedTemporaryFile(suffix=".pdf", delete=False) as tmp:
            tmp.write(pdf)
            tmp_path = Path(tmp.name)
            try:
                if sys.platform == "darwin":  # macOS
                    subprocess.run(["/usr/bin/open", tmp_path], check=True)  # noqa: S603
                elif sys.platform == "win32":  # Windows
                    os.startfile(tmp_path)  # type: ignore  # noqa: S606
                else:  # Linux and others
                    subprocess.run(["xdg-open", tmp_path], check=True)  # noqa: S607, S603

                # Give the viewer some time to open the file

                time.sleep(2)
            finally:
                # Clean up temp file after viewer has opened it
                tmp_path.unlink(missing_ok=True)



"""Tests for web utilities functionality."""

import asyncio
from unittest.mock import patch

import pytest
from httpx import TimeoutException

from starbridge import __project_name__
from starbridge.web import RobotForbiddenError
from starbridge.web.utils import (
    _ensure_allowed_to_crawl,
    get_additional_context_for_url,
)

GET_TEST_URL = "https://starbridge.readthedocs.io/en/latest/"
HTTPX_ASYNC_CLIENT_GET = "httpx.AsyncClient.get"
TIMEOUT_MESSAGE = "Connection timed out"
LLMS_TXT_URL = "https://docs.anthropic.com"
LLMS_TXT = "llms_txt"
LLMS_FULL_TXT = "llms-full.txt"
LLMS_DUMY_CONTENT = "llms content"


def test_web_utils_robots_disallowed_on_timeout() -> None:
    """Check disallowing on robots.txt timing out."""
    with pytest.raises(RobotForbiddenError), patch(HTTPX_ASYNC_CLIENT_GET) as mock_get:
        mock_get.side_effect = TimeoutException(TIMEOUT_MESSAGE)
        asyncio.run(_ensure_allowed_to_crawl(GET_TEST_URL, __project_name__))


def test_web_utils_robots_disallowed_on_401() -> None:
    """Check disallowing on robots.txt forbidden."""
    with pytest.raises(RobotForbiddenError), patch(HTTPX_ASYNC_CLIENT_GET) as mock_get:
        mock_get.return_value.status_code = 401
        asyncio.run(_ensure_allowed_to_crawl(GET_TEST_URL, __project_name__))


def test_web_utils_robots_allowed_on_404() -> None:
    """Check allowing on robots.txt not found."""
    with patch(HTTPX_ASYNC_CLIENT_GET) as mock_get:
        mock_get.return_value.status_code = 404
        # No exception should be raised
        asyncio.run(_ensure_allowed_to_crawl(GET_TEST_URL, __project_name__))


def test_web_utils_context_success() -> None:
    """Check context includes llms_txt if exists."""
    context = asyncio.run(
        get_additional_context_for_url(LLMS_TXT_URL, __project_name__),
    )
    assert any(ctx.type == "llms_txt" for ctx in context)


def test_web_utils_context_empty_on_404() -> None:
    """Check context empty if llms.txt not found."""
    with patch(HTTPX_ASYNC_CLIENT_GET) as mock_get:
        mock_get.return_value.status_code = 404
        # No exception should be raised
        context = asyncio.run(
            get_additional_context_for_url(LLMS_TXT_URL, __project_name__),
        )
        assert len(context) == 0


def test_web_utils_context_empty_on_timeout() -> None:
    """Check context empty on download of llms.txt timeout."""
    with patch(HTTPX_ASYNC_CLIENT_GET) as mock_get:
        mock_get.side_effect = TimeoutException(TIMEOUT_MESSAGE)
        # No exception should be raised
        context = asyncio.run(
            get_additional_context_for_url(LLMS_TXT_URL, __project_name__),
        )
        assert len(context) == 0


def test_web_utils_context_empty_on_full_timeout() -> None:
    """Check context empty on download of llms-full.txt and llms.txt timeout."""
    with patch(HTTPX_ASYNC_CLIENT_GET) as mock_get:
        mock_get.side_effect = TimeoutException(TIMEOUT_MESSAGE)
        # No exception should be raised
        context = asyncio.run(
            get_additional_context_for_url(LLMS_TXT_URL, __project_name__, full=True),
        )
        assert len(context) == 0


def test_web_utils_context_fallback_to_non_full() -> None:
    """Check context contains llms.txt even if llms-full.txt not found."""

    class MockResponse:
        def __init__(self, status_code, text="") -> None:
            self.status_code = status_code
            self.text = text

    def mock_get_side_effect(url, **kwargs):
        if LLMS_FULL_TXT in url:
            return MockResponse(404)
        return MockResponse(200, LLMS_DUMY_CONTENT)

    with patch(HTTPX_ASYNC_CLIENT_GET) as mock_get:
        mock_get.side_effect = mock_get_side_effect
        context = asyncio.run(
            get_additional_context_for_url(LLMS_TXT_URL, __project_name__, full=True),
        )
        assert len(context) >= 1
        llms_txt_context = next(
            (ctx for ctx in context if ctx.type == "llms_txt"),
            None,
        )
        assert llms_txt_context is not None
        assert str(llms_txt_context.url) == "https://docs.anthropic.com/llms.txt"
        assert llms_txt_context.text == LLMS_DUMY_CONTENT



"""Handles core services operations."""

from starbridge import (
    __is_development_mode__,
    __is_running_in_container__,
    __project_name__,
    __project_path__,
    __version__,
)
from starbridge.mcp import MCPBaseService, MCPContext, MCPServer, mcp_tool
from starbridge.utils import (
    Health,
    get_logger,
    get_process_info,
    get_starbridge_env,
)

logger = get_logger(__name__)


class Service(MCPBaseService):
    """Service class for core services if starbridge."""

    @mcp_tool()
    def health(self, context: MCPContext | None = None) -> Health:  # noqa: ARG002, PLR6301
        """
        Check the health of the core service of Starbridge.

        Args:
            context (MCPContext, optional): The context of the request. Defaults to None.

        Returns:
            Health: The health status of the core service

        """
        return Health(status=Health.Status.UP)  # We are up always

    @mcp_tool()
    def info(self, context: MCPContext | None = None) -> dict:  # noqa: ARG002, PLR6301
        """
        Get info about the environment starbridge is running in and all services.

        Args:
            context (MCPContext, optional): The context of the request. Defaults to None.

        Returns:
            dict: Information about the Starbridge environment and its services

        """
        rtn = {
            "name": __project_name__,
            "version": __version__,
            "path": __project_path__,
            "is_development_mode": __is_development_mode__,
            "is_running_in_container": __is_running_in_container__,
            "env": get_starbridge_env(),
            "process": get_process_info().model_dump(),
        }

        # Auto-discover and get info from all services
        for service_class in MCPServer.service_classes():
            if service_class is Service:  # Skip self
                continue
            service = service_class()
            service_name = service.__class__.__module__.split(".")[1]
            rtn[service_name] = service.info()

        return rtn



"""Tests for web types functionality."""

import pytest
from pydantic import AnyHttpUrl, ValidationError

from starbridge.web import Resource

EXAMPLE_COM = "https://example.com"


def test_web_types_resource_exactly_one() -> None:
    """Test that Resource validation requires exactly one field to be set."""
    with pytest.raises(ValidationError):
        Resource(
            url=AnyHttpUrl(EXAMPLE_COM),
            type="invalid",
            text="Hello World",
            blob=b"\0",
        )
    with pytest.raises(ValidationError):
        Resource(url=AnyHttpUrl(EXAMPLE_COM), type="invalid", text=None, blob=None)
    Resource(
        url=AnyHttpUrl(EXAMPLE_COM),
        type="invalid",
        text="Hello World",
        blob=None,
    )
    Resource(url=AnyHttpUrl(EXAMPLE_COM), type="invalid", text=None, blob=b"\0")



"""Nox configuration for test automation and development tasks."""

import json
import os
from pathlib import Path

import nox

nox.options.reuse_existing_virtualenvs = True
nox.options.default_venv_backend = "uv"

NOT_SKIP_WITH_ACT = "not skip_with_act"
JUNIT_XML = "--junitxml=junit.xml"


def _setup_venv(session: nox.Session, all_extras: bool = True) -> None:
    """Install dependencies for the given session using uv."""
    args = ["uv", "sync", "--frozen"]
    if all_extras:
        args.append("--all-extras")
    session.run_install(
        *args,
        env={
            "UV_PROJECT_ENVIRONMENT": session.virtualenv.location,
            "UV_PYTHON": str(session.python),
        },
    )


def _is_act_environment() -> bool:
    """Check if running in GitHub ACT environment.

    Returns:
        bool: True if running in ACT environment, False otherwise.
    """
    return os.environ.get("GITHUB_WORKFLOW_RUNTIME") == "ACT"


@nox.session(python=["3.13"])
def lint(session: nox.Session) -> None:
    """Run code formatting checks, linting, and static type checking."""
    _setup_venv(session)
    session.run("ruff", "check", ".")
    session.run(
        "ruff",
        "format",
        "--check",
        ".",
    )
    # session.run("mypy", "src")  # noqa: ERA001


@nox.session(python=["3.13"])
def docs(session: nox.Session) -> None:
    """Build documentation and concatenate README."""
    _setup_venv(session)
    # Concatenate README files
    header = Path("_readme_header.md").read_text(encoding="utf-8")
    main = Path("_readme_main.md").read_text(encoding="utf-8")
    footer = Path("_readme_footer.md").read_text(encoding="utf-8")
    readme_content = f"{header}\n\n{main}\n\n{footer}"
    Path("README.md").write_text(readme_content, encoding="utf-8")
    # Build docs
    session.run("make", "-C", "docs", "clean", external=True)
    session.run("make", "-C", "docs", "html", external=True)


@nox.session(python=["3.13"])
def audit(session: nox.Session) -> None:
    """Run security audit and license checks."""
    _setup_venv(session)
    session.run("pip-audit", "-f", "json", "-o", "vulnerabilities.json")
    session.run("jq", ".", "vulnerabilities.json", external=True)
    session.run("pip-licenses", "--format=json", "--output-file=licenses.json")
    session.run("jq", ".", "licenses.json", external=True)
    # Read and parse licenses.json
    licenses_data = json.loads(Path("licenses.json").read_text(encoding="utf-8"))

    licenses_inverted: dict[str, list[dict[str, str]]] = {}
    licenses_inverted = {}
    for pkg in licenses_data:
        license_name = pkg["License"]
        package_info = {"Name": pkg["Name"], "Version": pkg["Version"]}

        if license_name not in licenses_inverted:
            licenses_inverted[license_name] = []
        licenses_inverted[license_name].append(package_info)

    # Write inverted data
    Path("licenses-inverted.json").write_text(
        json.dumps(licenses_inverted, indent=2),
        encoding="utf-8",
    )
    session.run("jq", ".", "licenses-inverted.json", external=True)
    session.run("cyclonedx-py", "environment", "-o", "sbom.json")
    session.run("jq", ".", "sbom.json", external=True)


@nox.session(python=["3.11", "3.12", "3.13"])
def test(session: nox.Session) -> None:
    """Run tests with pytest."""
    _setup_venv(session)
    session.run("rm", "-rf", ".coverage", external=True)

    # Build pytest arguments with skip_with_act filter if needed
    pytest_args = ["pytest", "--disable-warnings", JUNIT_XML, "-n", "auto", "--dist", "loadgroup"]
    if _is_act_environment():
        pytest_args.extend(["-k", NOT_SKIP_WITH_ACT])
    pytest_args.extend(["-m", "not sequential"])

    session.run(*pytest_args)

    # Sequential tests
    sequential_args = [
        "pytest",
        "--cov-append",
        "--disable-warnings",
        JUNIT_XML,
        "-n",
        "auto",
        "--dist",
        "loadgroup",
    ]
    if _is_act_environment():
        sequential_args.extend(["-k", NOT_SKIP_WITH_ACT])
    sequential_args.extend(["-m", "sequential"])

    session.run(*sequential_args)

    session.run(
        "bash",
        "-c",
        (
            "docker compose ls --format json | jq -r '.[].Name' | "
            "grep ^pytest | xargs -I {} docker compose -p {} down --remove-orphans"
        ),
        external=True,
    )


@nox.session(python=["3.11", "3.12", "3.13"])
def test_no_extras(session: nox.Session) -> None:
    """Run test sessions without extra dependencies."""
    _setup_venv(session, all_extras=False)

    no_extras_args = ["pytest", "--cov-append", "--disable-warnings", JUNIT_XML, "-n", "1"]
    if _is_act_environment():
        no_extras_args.extend(["-k", NOT_SKIP_WITH_ACT])
    no_extras_args.extend(["-m", "no_extras"])

    session.run(*no_extras_args)

    session.run(
        "bash",
        "-c",
        (
            "docker compose ls --format json | jq -r '.[].Name' | "
            "grep ^pytest | xargs -I {} docker compose -p {} down --remove-orphans"
        ),
        external=True,
    )


@nox.session(python=["3.13"], default=False)
def setup_dev(session: nox.Session) -> None:
    """Setup dev environment post project creation."""
    _setup_venv(session)
    session.run("ruff", "format", ".", external=True)
    git_dir = Path(".git")
    if git_dir.is_dir():
        session.run("echo", "found .git directory", external=True)
        session.run("touch", ".act-env-secret", external=True)
        session.run("pre-commit", "install", external=True)
        with Path(".secrets.baseline").open("w", encoding="utf-8") as out:
            session.run("detect-secrets", "scan", stdout=out, external=True)
        session.run("git", "add", ".", external=True)
        try:
            session.run("pre-commit", external=True)
        except Exception:  # noqa: BLE001
            session.log("pre-commit run failed, continuing anyway")
        session.run("git", "add", ".", external=True)



"""Tests for core environment handling."""

import os
import shutil
import subprocess
import sys
from collections.abc import Generator
from pathlib import Path
from typing import Never
from unittest.mock import patch

import pytest
import typer
from typer.testing import CliRunner

from starbridge.cli import cli


@pytest.fixture
def runner() -> CliRunner:
    """Get a Click CLI test runner."""
    return CliRunner()


@pytest.fixture(autouse=True)
def backup_env() -> Generator[None, None, None]:
    """Fixture to automatically backup and restore .env file."""
    env_path = Path(__file__).parent.parent / ".env"
    bak_path = Path(__file__).parent.parent / ".env.bak"
    if env_path.is_file():
        shutil.copy2(env_path, bak_path)
    yield
    if bak_path.is_file():
        shutil.move(bak_path, env_path)


def test_core_env_args_passed(runner) -> None:
    """Check --env can override environment for some commands."""

    def mock_asyncio_run(x) -> Never:
        raise typer.Exit(42)

    with patch("asyncio.run", side_effect=mock_asyncio_run):
        result = runner.invoke(
            cli,
            [
                "mcp",
                "serve",
                "--env",
                'STARBRIDGE_ATLASSIAN_URL="https://test.com"',
                "--env",
                "OTHER_ENV=4711",
                "--env",
                "VALUE_ERROR",
            ],
        )
        assert result.exit_code == 42

    with patch("asyncio.run", side_effect=mock_asyncio_run):
        result = runner.invoke(
            cli,
            ["--env", 'STARBRIDGE_ATLASSIAN_URL="https://test.com"'],
        )
        assert result.exit_code == 42


def test_core_env_args_fail(runner) -> None:
    """Check --env not supported for all commands."""
    result = subprocess.run(
        ["starbridge", "info", "--env", 'STARBRIDGE_LOG_LEVEL="DEBUG"'],
        capture_output=True,
        text=True,
        check=False,
    )
    assert "No such option" in result.stderr
    assert result.returncode == 2


@pytest.mark.sequential
def test_core_dot_env_validated(runner) -> None:
    """Check missing entry in .env leads to validation error."""
    result = runner.invoke(cli, ["health"])
    assert result.exit_code == 0

    # Read .env, remove STARBRIDGE_ATLASSIAN_URL line and write back
    env_path = Path(__file__).parent.parent.parent / ".env"
    # Backup .env using Pathlib
    bak_path = Path(__file__).parent.parent.parent / ".env.bak"
    Path(bak_path).write_text(Path(env_path).read_text(encoding="utf-8"), encoding="utf-8")

    with open(env_path, encoding="utf-8") as f:
        lines = f.readlines()

    with open(env_path, "w", encoding="utf-8") as f:
        for line in lines:
            if not line.startswith("STARBRIDGE_ATLASSIAN_URL"):
                f.write(line)
    os.environ.pop("STARBRIDGE_ATLASSIAN_URL", None)

    result = runner.invoke(cli, ["health"])
    Path(env_path).write_text(Path(bak_path).read_text(encoding="utf-8"), encoding="utf-8")
    assert result.exit_code == 78
    assert "STARBRIDGE_ATLASSIAN_URL: Field required" in result.output


def test_parse_env_args() -> None:
    """Test parsing of environment variables from command line arguments."""
    from starbridge import _parse_env_args

    original_argv = sys.argv
    original_env = os.environ.copy()

    try:
        # Test valid STARBRIDGE_ prefixed vars
        sys.argv = [
            "starbridge",
            "--env",
            'STARBRIDGE_TEST="value"',
            "-e",
            "STARBRIDGE_OTHER=123",
            "--env",
            'STARBRIDGE_QUOTED="quoted value"',
        ]
        _parse_env_args()
        assert os.environ["STARBRIDGE_TEST"] == "value"
        assert os.environ["STARBRIDGE_OTHER"] == "123"
        assert os.environ["STARBRIDGE_QUOTED"] == "quoted value"

        # Test non-STARBRIDGE vars are ignored
        sys.argv = ["starbridge", "--env", "OTHER_VAR=ignored"]
        _parse_env_args()
        assert "OTHER_VAR" not in os.environ

        # Test malformed vars are skipped
        sys.argv = ["starbridge", "--env", "STARBRIDGE_INVALID"]
        _parse_env_args()
        assert "STARBRIDGE_INVALID" not in os.environ

    finally:
        sys.argv = original_argv
        os.environ.clear()
        os.environ.update(original_env)



"""Tests for MCP context functionality."""

import pytest

from starbridge.instrumentation import logfire_initialize
from starbridge.mcp import MCPContext

logfire_initialize()


def test_mcp_context_read_request_fails_outside_server() -> None:
    """Test listing of tools from the server."""
    context = MCPContext(request_context=None)
    # Assert that calling context.request_context() raises a RuntimeError
    with pytest.raises(RuntimeError):
        context.request_context  # noqa: B018



"""Test utility functions of the Starbridge package."""

import os

from mcp import StdioServerParameters
from mcp.client.stdio import get_default_environment

PYPROJECT_TOML = "pyproject.toml"
DOT_COVERAGE = ".coverage"


def _server_parameters(mocks: list[str] | None = None) -> StdioServerParameters:
    """Create server parameters with coverage enabled."""
    env = dict(get_default_environment())
    # Add coverage config to subprocess
    env.update({
        "COVERAGE_PROCESS_START": PYPROJECT_TOML,
        "COVERAGE_FILE": os.getenv("COVERAGE_FILE", DOT_COVERAGE),
    })
    if (mocks is not None) and mocks:
        env.update({"MOCKS": ",".join(mocks)})

    return StdioServerParameters(
        command="uv",
        args=["run", "starbridge"],
        env=env,
    )



"""Command-line interface utilities for the starbridge package."""

import typer

from .di import (
    locate_implementations,
)


def prepare_cli(cli: typer.Typer, epilog: str) -> None:
    """
    Dynamically locate, register and prepare subcommands.

    Args:
        cli (typer.Typer): Typer instance
        epilog (str): Epilog to add

    """
    for _cli in locate_implementations(typer.Typer):
        if _cli != cli:
            cli.add_typer(_cli)

    # add epilog for all subcommands
    _add_epilog_recursively(cli, epilog)

    # add no_args_is_help for all subcommands
    _no_args_is_help_recursively(cli)


def _add_epilog_recursively(cli: typer.Typer, epilog: str) -> None:
    """
    Add epilog to all typers in the tree.

    Args:
        cli (typer.Typer): Typer instance
        epilog (str): Epilog to add

    """
    cli.info.epilog = epilog
    for group in cli.registered_groups:
        if isinstance(group, typer.models.TyperInfo):
            typer_instance = group.typer_instance
            if (typer_instance is not cli) and typer_instance:
                _add_epilog_recursively(typer_instance, epilog)
    for command in cli.registered_commands:
        if isinstance(command, typer.models.CommandInfo):
            command.epilog = cli.info.epilog


def _no_args_is_help_recursively(cli: typer.Typer) -> None:
    """
    Add epilog to all typers in the tree.

    Args:
        cli (typer.Typer): Typer instance

    """
    for group in cli.registered_groups:
        if isinstance(group, typer.models.TyperInfo):
            group.no_args_is_help = True
            typer_instance = group.typer_instance
            if (typer_instance is not cli) and typer_instance:
                _no_args_is_help_recursively(typer_instance)



"""Tests for Claude service functionality."""

from collections.abc import Generator
from unittest.mock import patch

import pytest

SUBPROCESS_RUN = "subprocess.run"


class TestClaudeService:
    """Test cases for the Claude service."""

    @pytest.fixture
    @staticmethod
    def mock_darwin() -> Generator[None, None, None]:
        """
        Mock platform.system to return Darwin.

        Yields:
            None: Yields control back after mocking.

        """
        with patch("platform.system", return_value="Darwin"):
            yield



"""Command-line interface module for Starbridge, providing various commands for service management and configuration."""

import sys
from pathlib import Path
from typing import Annotated

import typer

from starbridge import (
    __is_development_mode__,
    __version__,
)
from starbridge.claude import Service as ClaudeService
from starbridge.claude import generate_mcp_server_config
from starbridge.mcp import MCPServer
from starbridge.utils import (
    console,
    get_logger,
    prepare_cli,
    prompt_for_env,
)

from .service import Service

logger = get_logger(__name__)

cli = typer.Typer(
    name="Starbridge CLI",
)


@cli.callback(invoke_without_command=True)
def main(
    ctx: typer.Context,
    host: Annotated[
        str | None,
        typer.Option(
            help="Host to run the server on",
        ),
    ] = None,
    port: Annotated[
        int | None,
        typer.Option(
            help="Port to run the server on",
        ),
    ] = None,
    debug: Annotated[
        bool,
        typer.Option(
            help="Debug mode",
        ),
    ] = True,
    env: Annotated[  # noqa: ARG001
        list[str] | None,
        typer.Option(
            "--env",
            help=(
                "Environment variables in key=value format. Can be used multiple times in one call. "
                "Only STARBRIDGE_ prefixed vars are evaluated. Example --env "
                'STARBRIDGE_ATLASSIAN_URL="https://your-domain.atlassian.net" --env '
                'STARBRIDGE_ATLASSIAN_EMAIL="YOUR_EMAIL"'
            ),
        ),
    ] = None,
) -> None:
    """
    Run MCP Server - alias for 'mcp serve'.

    Args:
        ctx (typer.Context): Typer context
        host (str): Host to run the server on
        port (int): Port to run the server on
        debug (bool): Debug mode
        env (list[str]): Environment variables in key=value format. Can be used multiple times in one call.
            Only STARBRIDGE_ prefixed vars are used. Example --env
            'STARBRIDGE_ATLASSIAN_URL="https://your-domain.atlassian.net" --env STARBRIDGE_ATLASSIAN_EMAIL="YOUR_EMAIL"'

    """
    # --env parsed in main __init__.py
    if ctx.invoked_subcommand is None:
        MCPServer.serve(host, port, debug)


@cli.command()
def health(json: Annotated[bool, typer.Option(help="Output health as JSON")] = False) -> None:
    """Check health of services and their dependencies."""
    health = MCPServer().health()
    if not health.healthy:
        logger.warning("health: %s", health)
    if json:
        console.print(health.model_dump_json())
    else:
        console.print(health)


@cli.command()
def info() -> None:
    """Info about Starbridge and it's environment."""
    data = Service().info()
    console.print_json(data=data)
    logger.debug(data)


@cli.command()
def create_dot_env() -> None:
    """
    Create .env file for Starbridge. You will be prompted for settings.

    Raises:
        RuntimeError: If not running in development mode.

    """
    if not __is_development_mode__:
        msg = "This command is only available in development mode"
        raise RuntimeError(msg)
    with Path(".env").open("w", encoding="utf-8") as f:
        for key, value in iter(prompt_for_env().items()):
            f.write(f"{key}={value}\n")
            f.write(f"{key}={value}\n")


@cli.command()
def install(
    restart_claude: Annotated[
        bool,
        typer.Option(
            help="Restart Claude Desktop application post installation",
        ),
    ] = ClaudeService.platform_supports_restart(),
    image: Annotated[
        str,
        typer.Option(
            help="Docker image to use for Starbridge. Only applies if started as container.",
        ),
    ] = "helmuthva/starbridge:latest",
) -> None:
    """
    Install starbridge within Claude Desktop application.

    Adds starbridge configuration and restarts Claude Desktop app.

    Args:
        restart_claude (bool): Restart Claude Desktop application post installation
        image (str): Docker image to use for Starbridge. Only applies if started as container

    """
    if ClaudeService.install_mcp_server(
        generate_mcp_server_config(prompt_for_env(), image),
        restart=restart_claude,
    ):
        console.print("Starbridge installed with Claude Desktop application.")
        if not restart_claude:
            console.print(
                "Please restart Claude Desktop application to complete the installation.",
            )
    else:
        console.print("Starbridge was already installed", style="warning")


@cli.command()
def uninstall(
    restart_claude: Annotated[
        bool,
        typer.Option(
            help="Restart Claude Desktop application post installation",
        ),
    ] = ClaudeService.platform_supports_restart(),
) -> None:
    """
    Uninstall starbridge from Claude Desktop application.

    Removes starbridge configuration and restarts Claude Desktop app.

    Args:
        restart_claude (bool): Restart Claude Desktop application post installation

    """
    if ClaudeService.uninstall_mcp_server(restart=restart_claude):
        console.print("Starbridge uninstalled from Claude Destkop application.")
    else:
        console.print("Starbridge was no installed", style="warning")


prepare_cli(cli, f"⭐ Starbridge v{__version__} - built with love in Berlin 🐻")

if __name__ == "__main__":
    try:
        cli()
    except Exception as e:  # noqa: BLE001
        logger.critical("Fatal error occurred: %s", e)
        console.print(f"Fatal error occurred: {e}", style="error")
        sys.exit(1)
